searchNodes=[{"doc":"Convenience macros to work with Aja's data structures. Use import A to import everything, or import only the macros you need.","ref":"A.html","title":"A","type":"module"},{"doc":"Convenience operator to concatenate an enumerable right to a vector left . left has to be an A.Vector , right can be any Enumerable . It is just an alias for A.Vector.concat/2 . Only available on Elixir versions &gt;= 1.11. Examples iex&gt; import A iex&gt; vec ( 5 .. 1 ) ++ + vec ( [ :boom , nil ] ) vec ( [ 5 , 4 , 3 , 2 , 1 , :boom , nil ] ) iex&gt; vec ( 5 .. 1 ) ++ + 0 .. 3 vec ( [ 5 , 4 , 3 , 2 , 1 , 0 , 1 , 2 , 3 ] )","ref":"A.html#+++/2","title":"A.+++/2","type":"function"},{"doc":"Convenience macro to create or pattern match on A.OrdMap s. Use import A to use it, or import A, only: [ord: 1] . Creation examples iex&gt; ord ( %{ &quot;一&quot; =&gt; 1 , &quot;二&quot; =&gt; 2 , &quot;三&quot; =&gt; 3 } ) ord ( %{ &quot;一&quot; =&gt; 1 , &quot;二&quot; =&gt; 2 , &quot;三&quot; =&gt; 3 } ) iex&gt; ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } ) Pattern matching examples iex&gt; ord ( %{ b : bat } ) = ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } ) ; bat &quot;Bat&quot; Replace existing keys examples iex&gt; ordered = ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } ) iex&gt; ord ( %{ ordered | b : &quot;Buffalo&quot; } ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Buffalo&quot; , c : &quot;Cat&quot; } ) iex&gt; ord ( %{ ordered | z : &quot;Zebra&quot; } ) ** (KeyError) key :z not found in: ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})","ref":"A.html#ord/1","title":"A.ord/1","type":"macro"},{"doc":"Returns the size of an ord_map . It is implemented as a macro so that it can be used in guards. When used outside of a guard, it will just be replaced by a call to A.OrdMap.size/1 . When used in guards, it will fail if called on something else than an A.OrdMap . It is recommended to verify the type first. Runs in constant time. Examples iex&gt; import A iex&gt; ord_map = A.OrdMap . new ( a : 1 , b : 2 , c : 3 ) iex&gt; match? ( v when ord_size ( v ) &gt; 5 , ord_map ) false iex&gt; match? ( v when ord_size ( v ) &lt; 5 , ord_map ) true iex&gt; ord_size ( ord_map ) 3","ref":"A.html#ord_size/1","title":"A.ord_size/1","type":"macro"},{"doc":"A sigil to build IO data and avoid string concatenation. Use import A to use it, or import A, only: [sigil_i: 2] . This sigil provides a faster version of string interpolation which: will build a list with all chunks instead of concatenating them as a string uses A.IO.to_iodata/1 on interpolated values instead of to_string/1 , which: will keep lists untouched, without any validation or transformation will cast anything else using to_string/1 Works with both IO data and Chardata . See their respective documentation for more information. Examples iex&gt; ~i&quot;atom: \#{ :foo } , charlist: \#{ &#39;abc&#39; } , number: \#{ 12 + 2.35 } \\n &quot; [ &quot;atom: &quot; , &quot;foo&quot; , &quot;, charlist: &quot; , &#39;abc&#39; , &quot;, number: &quot; , &quot;14.35&quot; , 10 ] iex&gt; ~i&quot;abc \#{ [ &#39;def&#39; | &quot;ghi&quot; ] } &quot; [ &quot;abc&quot; , [ &#39;def&#39; | &quot;ghi&quot; ] ] iex&gt; ~i&quot;Giorno Giovanna&quot; &quot;Giorno Giovanna&quot; IO data can often be used as is without ever generating the corresponding string. If needed however, IO data can be cast as a string using IO.iodata_to_binary/1 , and chardata using List.to_string/1 . In most cases, both should be the same: iex&gt; IO . iodata_to_binary ( ~i&quot;atom: \#{ :foo } , charlist: \#{ &#39;abc&#39; } , number: \#{ 12 + 2.35 } \\n &quot; ) &quot;atom: foo, charlist: abc, number: 14.35 \\n &quot; iex&gt; List . to_string ( ~i&quot;abc \#{ [ &#39;def&#39; | &quot;ghi&quot; ] } &quot; ) &quot;abcdefghi&quot; Those are the exact same values returned by a regular string interpolation, without the ~i sigil: iex&gt; &quot;atom: \#{ :foo } , charlist: \#{ &#39;abc&#39; } , number: \#{ 12 + 2.35 } \\n &quot; &quot;atom: foo, charlist: abc, number: 14.35 \\n &quot; iex&gt; &quot;abc \#{ [ &#39;def&#39; | &quot;ghi&quot; ] } &quot; &quot;abcdefghi&quot;","ref":"A.html#sigil_i/2","title":"A.sigil_i/2","type":"macro"},{"doc":"Convenience macro to create or pattern match on A.Vector s. Examples iex&gt; import A iex&gt; vec ( [ 1 , 2 , 3 ] ) vec ( [ 1 , 2 , 3 ] ) iex&gt; vec ( first ||| last ) = A.Vector . new ( 0 .. 99_999 ) ; { first , last } { 0 , 99999 } iex&gt; vec ( [ 1 , 2 , var , _ , _ , _ ] ) = A.Vector . new ( 1 .. 6 ) ; var 3 iex&gt; vec ( [ _ , _ , _ ] ) = A.Vector . new ( 1 .. 6 ) ** (MatchError) no match of right hand side value: vec([1, 2, 3, 4, 5, 6]) It also supports ranges with constant values: iex&gt; vec ( 0 .. 4 ) = A.Vector . new ( 0 .. 4 ) vec ( [ 0 , 1 , 2 , 3 , 4 ] ) Variable lists or dynamic ranges cannot be passed: vec ( my_list ) # invalid vec ( 1 .. n ) # invalid Explanation The vec/1 macro generates the AST at compile time instead of building the vector at runtime. This can speedup the instanciation of vectors of known size.","ref":"A.html#vec/1","title":"A.vec/1","type":"macro"},{"doc":"Returns the size of a vector . It is implemented as a macro so that it can be used in guards. When used outside of a guard, it will just be replaced by a call to A.Vector.size/1 . When used in guards, it will fail if called on something else than an A.Vector . It is recommended to verify the type first. Runs in constant time. Examples iex&gt; import A iex&gt; match? ( v when vec_size ( v ) &gt; 20 , A.Vector . new ( 1 .. 10 ) ) false iex&gt; match? ( v when vec_size ( v ) &lt; 5 , A.Vector . new ( [ 1 , 2 , 3 ] ) ) true iex&gt; vec_size ( A.Vector . new ( [ 1 , 2 , 3 ] ) ) 3","ref":"A.html#vec_size/1","title":"A.vec_size/1","type":"macro"},{"doc":"Drop-in replacement for the Enum module, optimized to work with Aja's data structures such as A.Vector . It currently only covers a subset of Enum , but A.Enum aims to completely mirror the API of Enum , and should behave exactly the same for any type of Enumerable . The only expected difference should be a significant increase in performance for Aja structures. Rationale Structures such as A.Vector or A.OrdMap are implementing the Enumerable protocol, which means they can be used directly with the Enum module. The Enumerable protocol however comes with its overhead and is strongly limited in terms of performance. On the other hand, A.Enum provides hand-crafted highly-optimized functions that fully take advantage of immutable vectors. The speedup can easily reach more than a factor 10 compared to Enum used on non-list structures, and sometimes even be noticeably faster than Enum used over lists. One of the main reasons to adopt a specific data structure is the performance. Using vectors with Enum would defeat the purpose, hence the introduction of A.Enum . iex&gt; vector = A.Vector . new ( 1 .. 10000 ) iex&gt; Enum . sum ( vector ) # slow 50005000 iex&gt; A.Enum . sum ( vector ) # same result, much faster 50005000","ref":"A.Enum.html","title":"A.Enum","type":"module"},{"doc":"Returns true if all elements in enumerable are truthy. When an element has a falsy value ( false or nil ) iteration stops immediately and false is returned. In all other cases true is returned. Examples iex&gt; A.Enum . all? ( [ 1 , 2 , 3 ] ) true iex&gt; A.Enum . all? ( [ 1 , nil , 3 ] ) false iex&gt; A.Enum . all? ( [ ] ) true","ref":"A.Enum.html#all?/1","title":"A.Enum.all?/1","type":"function"},{"doc":"Returns true if fun.(element) is truthy for all elements in enumerable . Iterates over enumerable and invokes fun on each element. If fun ever returns a falsy value ( false or nil ), iteration stops immediately and false is returned. Otherwise, true is returned. Examples iex&gt; A.Enum . all? ( [ 2 , 4 , 6 ] , fn x -&gt; rem ( x , 2 ) == 0 end ) true iex&gt; A.Enum . all? ( [ 2 , 3 , 4 ] , fn x -&gt; rem ( x , 2 ) == 0 end ) false iex&gt; A.Enum . all? ( [ ] , fn _ -&gt; nil end ) true","ref":"A.Enum.html#all?/2","title":"A.Enum.all?/2","type":"function"},{"doc":"Returns true if at least one element in enumerable is truthy. When an element has a truthy value (neither false nor nil ) iteration stops immediately and true is returned. In all other cases false is returned. Examples iex&gt; A.Enum . any? ( [ false , false , false ] ) false iex&gt; A.Enum . any? ( [ false , true , false ] ) true iex&gt; A.Enum . any? ( [ ] ) false","ref":"A.Enum.html#any?/1","title":"A.Enum.any?/1","type":"function"},{"doc":"Returns true if fun.(element) is truthy for at least one element in enumerable . Iterates over the enumerable and invokes fun on each element. When an invocation of fun returns a truthy value (neither false nor nil ) iteration stops immediately and true is returned. In all other cases false is returned. Examples iex&gt; A.Enum . any? ( [ 2 , 4 , 6 ] , fn x -&gt; rem ( x , 2 ) == 1 end ) false iex&gt; A.Enum . any? ( [ 2 , 3 , 4 ] , fn x -&gt; rem ( x , 2 ) == 1 end ) true iex&gt; A.Enum . any? ( [ ] , fn x -&gt; x &gt; 0 end ) false","ref":"A.Enum.html#any?/2","title":"A.Enum.any?/2","type":"function"},{"doc":"Finds the element at the given index (zero-based). Mirrors Enum.at/3 with higher performance for Aja structures.","ref":"A.Enum.html#at/3","title":"A.Enum.at/3","type":"function"},{"doc":"Given an enumerable of enumerables, concatenates the enumerables into a single list. Mirrors Enum.concat/1 with higher performance for Aja structures.","ref":"A.Enum.html#concat/1","title":"A.Enum.concat/1","type":"function"},{"doc":"Concatenates the enumerable on the right with the enumerable on the left . Mirrors Enum.concat/2 with higher performance for Aja structures.","ref":"A.Enum.html#concat/2","title":"A.Enum.concat/2","type":"function"},{"doc":"Returns the size of the enumerable . Mirrors Enum.count/1 with higher performance for Aja structures.","ref":"A.Enum.html#count/1","title":"A.Enum.count/1","type":"function"},{"doc":"Returns the count of elements in the enumerable for which fun returns a truthy value. Mirrors Enum.count/2 with higher performance for Aja structures.","ref":"A.Enum.html#count/2","title":"A.Enum.count/2","type":"function"},{"doc":"Enumerates the enumerable , returning a list where all consecutive duplicated elements are collapsed to a single element. Mirrors Enum.dedup/1 with higher performance for Aja structures.","ref":"A.Enum.html#dedup/1","title":"A.Enum.dedup/1","type":"function"},{"doc":"Enumerates the enumerable , returning a list where all consecutive duplicated elements are collapsed to a single element. Mirrors Enum.dedup_by/2 with higher performance for Aja structures.","ref":"A.Enum.html#dedup_by/2","title":"A.Enum.dedup_by/2","type":"function"},{"doc":"Drops the amount of elements from the enumerable . Mirrors Enum.drop/2 with higher performance for Aja structures.","ref":"A.Enum.html#drop/2","title":"A.Enum.drop/2","type":"function"},{"doc":"Drops elements at the beginning of the enumerable while fun returns a truthy value. Mirrors Enum.drop_while/2 with higher performance for Aja structures.","ref":"A.Enum.html#drop_while/2","title":"A.Enum.drop_while/2","type":"function"},{"doc":"Invokes the given fun for each element in the enumerable . Mirrors Enum.each/2 with higher performance for Aja structures.","ref":"A.Enum.html#each/2","title":"A.Enum.each/2","type":"function"},{"doc":"Returns true if enumerable is empty, otherwise false . Mirrors Enum.empty?/1 with higher performance for Aja structures.","ref":"A.Enum.html#empty?/1","title":"A.Enum.empty?/1","type":"function"},{"doc":"Finds the element at the given index (zero-based). Mirrors Enum.fetch/2 with higher performance for Aja structures.","ref":"A.Enum.html#fetch/2","title":"A.Enum.fetch/2","type":"function"},{"doc":"Finds the element at the given index (zero-based). Mirrors Enum.fetch!/2 with higher performance for Aja structures.","ref":"A.Enum.html#fetch!/2","title":"A.Enum.fetch!/2","type":"function"},{"doc":"Filters the enumerable , i.e. returns only those elements for which fun returns a truthy value. Mirrors Enum.filter/2 with higher performance for Aja structures.","ref":"A.Enum.html#filter/2","title":"A.Enum.filter/2","type":"function"},{"doc":"Returns the first element for which fun returns a truthy value. If no such element is found, returns default . Mirrors Enum.find/3 with higher performance for Aja structures.","ref":"A.Enum.html#find/3","title":"A.Enum.find/3","type":"function"},{"doc":"Similar to find/3 , but returns the index (zero-based) of the element instead of the element itself. Mirrors Enum.find_index/2 with higher performance for Aja structures.","ref":"A.Enum.html#find_index/2","title":"A.Enum.find_index/2","type":"function"},{"doc":"Similar to find/3 , but returns the value of the function invocation instead of the element itself. Mirrors Enum.find_value/3 with higher performance for Aja structures.","ref":"A.Enum.html#find_value/3","title":"A.Enum.find_value/3","type":"function"},{"doc":"Maps the given fun over enumerable and flattens the result. Mirrors Enum.flat_map/2 with higher performance for Aja structures.","ref":"A.Enum.html#flat_map/2","title":"A.Enum.flat_map/2","type":"function"},{"doc":"Returns a map with keys as unique elements of enumerable and values as the count of every element. Mirrors Enum.frequencies/1 with higher performance for Aja structures.","ref":"A.Enum.html#frequencies/1","title":"A.Enum.frequencies/1","type":"function"},{"doc":"Returns a map with keys as unique elements given by key_fun and values as the count of every element. Mirrors Enum.frequencies_by/2 with higher performance for Aja structures.","ref":"A.Enum.html#frequencies_by/2","title":"A.Enum.frequencies_by/2","type":"function"},{"doc":"Splits the enumerable into groups based on key_fun . Mirrors Enum.group_by/3 with higher performance for Aja structures.","ref":"A.Enum.html#group_by/3","title":"A.Enum.group_by/3","type":"function"},{"doc":"Intersperses separator between each element of the given enumerable . Mirrors Enum.intersperse/2 with higher performance for Aja structures.","ref":"A.Enum.html#intersperse/2","title":"A.Enum.intersperse/2","type":"function"},{"doc":"Inserts the given enumerable into a collectable . Mirrors Enum.into/2 with higher performance for Aja structures.","ref":"A.Enum.html#into/2","title":"A.Enum.into/2","type":"function"},{"doc":"Inserts the given enumerable into a collectable according to the transform function. Mirrors Enum.into/3 with higher performance for Aja structures.","ref":"A.Enum.html#into/3","title":"A.Enum.into/3","type":"function"},{"doc":"Joins the given enumerable into a string using joiner as a separator. Mirrors Enum.join/2 with higher performance for Aja structures.","ref":"A.Enum.html#join/2","title":"A.Enum.join/2","type":"function"},{"doc":"Returns a list where each element is the result of invoking fun on each corresponding element of enumerable . Mirrors Enum.map/2 with higher performance for Aja structures.","ref":"A.Enum.html#map/2","title":"A.Enum.map/2","type":"function"},{"doc":"Maps and intersperses the given enumerable in one pass. Mirrors Enum.map_intersperse/3 with higher performance for Aja structures.","ref":"A.Enum.html#map_intersperse/3","title":"A.Enum.map_intersperse/3","type":"function"},{"doc":"Maps and joins the given enumerable in one pass. Mirrors Enum.map_join/3 with higher performance for Aja structures.","ref":"A.Enum.html#map_join/3","title":"A.Enum.map_join/3","type":"function"},{"doc":"Invokes the given function to each element in the enumerable to reduce it to a single element, while keeping an accumulator. Mirrors Enum.map_reduce/3 with higher performance for Aja structures.","ref":"A.Enum.html#map_reduce/3","title":"A.Enum.map_reduce/3","type":"function"},{"doc":"Returns the maximal element in the enumerable according to Erlang's term ordering. Mirrors Enum.max/3 with higher performance for Aja structures.","ref":"A.Enum.html#max/3","title":"A.Enum.max/3","type":"function"},{"doc":"Returns the maximal element in the enumerable as calculated by the given fun . Mirrors Enum.max_by/4 with higher performance for Aja structures.","ref":"A.Enum.html#max_by/4","title":"A.Enum.max_by/4","type":"function"},{"doc":"Checks if element exists within the enumerable . Just an alias for Enum.member?/2 , does not improve performance.","ref":"A.Enum.html#member?/2","title":"A.Enum.member?/2","type":"function"},{"doc":"Returns the minimal element in the enumerable according to Erlang's term ordering. Mirrors Enum.min/3 with higher performance for Aja structures.","ref":"A.Enum.html#min/3","title":"A.Enum.min/3","type":"function"},{"doc":"Returns the minimal element in the enumerable as calculated by the given fun . Mirrors Enum.min_by/4 with higher performance for Aja structures.","ref":"A.Enum.html#min_by/4","title":"A.Enum.min_by/4","type":"function"},{"doc":"Returns the product of all elements in the enumerable . Mirrors Enum.product/1 from Elixir 1.12. Raises ArithmeticError if enumerable contains a non-numeric value. Examples iex&gt; 1 .. 5 |&gt; A.Enum . product ( ) 120 iex&gt; [ ] |&gt; A.Enum . product ( ) 1","ref":"A.Enum.html#product/1","title":"A.Enum.product/1","type":"function"},{"doc":"Returns a random element of an enumerable . Mirrors Enum.random/1 with higher performance for Aja structures.","ref":"A.Enum.html#random/1","title":"A.Enum.random/1","type":"function"},{"doc":"Invokes fun for each element in the enumerable with the accumulator. Mirrors Enum.reduce/2 with higher performance for Aja structures.","ref":"A.Enum.html#reduce/2","title":"A.Enum.reduce/2","type":"function"},{"doc":"Invokes fun for each element in the enumerable with the accumulator. Mirrors Enum.reduce/3 with higher performance for Aja structures.","ref":"A.Enum.html#reduce/3","title":"A.Enum.reduce/3","type":"function"},{"doc":"Returns a list of elements in enumerable excluding those for which the function fun returns a truthy value. Mirrors Enum.reject/2 with higher performance for Aja structures.","ref":"A.Enum.html#reject/2","title":"A.Enum.reject/2","type":"function"},{"doc":"Returns a list of elements in enumerable in reverse order. Mirrors Enum.reverse/1 with higher performance for Aja structures.","ref":"A.Enum.html#reverse/1","title":"A.Enum.reverse/1","type":"function"},{"doc":"Reverses the elements in enumerable , concatenates the tail , and returns it as a list. Mirrors Enum.reverse/2 with higher performance for Aja structures.","ref":"A.Enum.html#reverse/2","title":"A.Enum.reverse/2","type":"function"},{"doc":"Applies the given function to each element in the enumerable , storing the result in a list and passing it as the accumulator for the next computation. Uses the first element in the enumerable as the starting value. Mirrors Enum.scan/2 with higher performance for Aja structures.","ref":"A.Enum.html#scan/2","title":"A.Enum.scan/2","type":"function"},{"doc":"Applies the given function to each element in the enumerable , storing the result in a list and passing it as the accumulator for the next computation. Uses the given acc as the starting value. Mirrors Enum.scan/3 with higher performance for Aja structures.","ref":"A.Enum.html#scan/3","title":"A.Enum.scan/3","type":"function"},{"doc":"Returns a list with the elements of enumerable shuffled. Mirrors Enum.shuffle/1 with higher performance for Aja structures.","ref":"A.Enum.html#shuffle/1","title":"A.Enum.shuffle/1","type":"function"},{"doc":"Returns a subset list of the given enumerable by index_range . Mirrors Enum.slice/2 with higher performance for Aja structures.","ref":"A.Enum.html#slice/2","title":"A.Enum.slice/2","type":"function"},{"doc":"Returns a subset list of the given enumerable , from start_index (zero-based) with amount number of elements if available. Mirrors Enum.slice/3 .","ref":"A.Enum.html#slice/3","title":"A.Enum.slice/3","type":"function"},{"doc":"Sorts the enumerable according to Erlang's term ordering. Mirrors Enum.sort/1 with higher performance for Aja structures.","ref":"A.Enum.html#sort/1","title":"A.Enum.sort/1","type":"function"},{"doc":"Sorts the enumerable by the given function. Mirrors Enum.sort/2 with higher performance for Aja structures.","ref":"A.Enum.html#sort/2","title":"A.Enum.sort/2","type":"function"},{"doc":"Sorts the mapped results of the enumerable according to the provided sorter function. Mirrors Enum.sort_by/3 with higher performance for Aja structures.","ref":"A.Enum.html#sort_by/3","title":"A.Enum.sort_by/3","type":"function"},{"doc":"Splits the enumerable into two enumerables, leaving count elements in the first one. Mirrors Enum.split/2 with higher performance for Aja structures.","ref":"A.Enum.html#split/2","title":"A.Enum.split/2","type":"function"},{"doc":"Splits enumerable in two at the position of the element for which fun returns a falsy value ( false or nil ) for the first time. Mirrors Enum.split_while/2 with higher performance for Aja structures.","ref":"A.Enum.html#split_while/2","title":"A.Enum.split_while/2","type":"function"},{"doc":"Splits the enumerable in two lists according to the given function fun . Mirrors Enum.split_with/2 with higher performance for Aja structures.","ref":"A.Enum.html#split_with/2","title":"A.Enum.split_with/2","type":"function"},{"doc":"Returns the sum of all elements. Mirrors Enum.sum/1 with higher performance for Aja structures.","ref":"A.Enum.html#sum/1","title":"A.Enum.sum/1","type":"function"},{"doc":"Takes an amount of elements from the beginning or the end of the enumerable . Mirrors Enum.take/2 with higher performance for Aja structures.","ref":"A.Enum.html#take/2","title":"A.Enum.take/2","type":"function"},{"doc":"Takes count random elements from enumerable . Mirrors Enum.take_random/2 with higher performance for Aja structures.","ref":"A.Enum.html#take_random/2","title":"A.Enum.take_random/2","type":"function"},{"doc":"Takes the elements from the beginning of the enumerable while fun returns a truthy value. Mirrors Enum.take_while/2 with higher performance for Aja structures.","ref":"A.Enum.html#take_while/2","title":"A.Enum.take_while/2","type":"function"},{"doc":"Converts enumerable to a list. Mirrors Enum.to_list/1 with higher performance for Aja structures.","ref":"A.Enum.html#to_list/1","title":"A.Enum.to_list/1","type":"function"},{"doc":"Enumerates the enumerable , removing all duplicated elements. Mirrors Enum.uniq/1 with higher performance for Aja structures.","ref":"A.Enum.html#uniq/1","title":"A.Enum.uniq/1","type":"function"},{"doc":"Enumerates the enumerable , by removing the elements for which function fun returned duplicate elements. Mirrors Enum.uniq_by/2 with higher performance for Aja structures.","ref":"A.Enum.html#uniq_by/2","title":"A.Enum.uniq_by/2","type":"function"},{"doc":"Opposite of zip/2 . Extracts two-element tuples from the given enumerable and groups them together. Mirrors Enum.unzip/1 with higher performance for Aja structures.","ref":"A.Enum.html#unzip/1","title":"A.Enum.unzip/1","type":"function"},{"doc":"Returns a list with with each element of enumerable wrapped in a tuple alongside its index. Mirrors Enum.with_index/2 (Elixir 1.12 version): may receive a function or an integer offset. If an integer offset is given, it will index from the given offset instead of from zero. If a function is given, it will index by invoking the function for each element and index (zero-based) of the enumerable . Examples iex&gt; A.Enum . with_index ( [ :a , :b , :c ] ) [ a : 0 , b : 1 , c : 2 ] iex&gt; A.Enum . with_index ( [ :a , :b , :c ] , 3 ) [ a : 3 , b : 4 , c : 5 ] iex&gt; A.Enum . with_index ( [ :a , :b , :c ] , fn element , index -&gt; { index , element } end ) [ { 0 , :a } , { 1 , :b } , { 2 , :c } ]","ref":"A.Enum.html#with_index/2","title":"A.Enum.with_index/2","type":"function"},{"doc":"Zips corresponding elements from two enumerables into one list of tuples. Mirrors Enum.zip/2 with higher performance for Aja structures.","ref":"A.Enum.html#zip/2","title":"A.Enum.zip/2","type":"function"},{"doc":"","ref":"A.Enum.html#t:index/0","title":"A.Enum.index/0","type":"type"},{"doc":"","ref":"A.Enum.html#t:t/1","title":"A.Enum.t/1","type":"type"},{"doc":"","ref":"A.Enum.html#t:value/0","title":"A.Enum.value/0","type":"type"},{"doc":"Some extra helper functions for working with IO data, that are not in the core IO module.","ref":"A.IO.html","title":"A.IO","type":"module"},{"doc":"Checks if IO data is empty in &quot;constant&quot; time. Should only need to loop until it finds one character or binary to stop, unlike IO.iodata_length(iodata) == 0 which needs to perform the complete loop to compute the length first. Examples iex&gt; A.IO . iodata_empty? ( [ &quot;&quot; , [ ] ] ) true iex&gt; A.IO . iodata_empty? ( &#39;a&#39; ) false iex&gt; A.IO . iodata_empty? ( [ &quot;a&quot; ] ) false iex&gt; A.IO . iodata_empty? ( [ &quot;&quot; , [ ] , [ &quot;&quot; | &quot;c&quot; ] ] ) false Rationale Even if IO.iodata_length/1 is a very efficient BIF implemented in C, it has a linear algorithmic complexity and can become slow if invoked on an IO list with many elements. This is not a far-fetched scenario, and a production use case can easily include &quot;big&quot; IO-lists with: JSON encoding to IO-data of long lists / nested objects loops within HTML templates","ref":"A.IO.html#iodata_empty?/1","title":"A.IO.iodata_empty?/1","type":"function"},{"doc":"Converts the argument to IO data according to the String.Chars protocol. Leaves lists untouched without any validation, calls to_string/1 on everything else. This is the function invoked in string interpolations within the i sigil . Works with both IO data and Chardata , depending on the type of the data parameter. Examples iex&gt; A.IO . to_iodata ( :foo ) &quot;foo&quot; iex&gt; A.IO . to_iodata ( 99 ) &quot;99&quot; iex&gt; A.IO . to_iodata ( [ &quot;abc&quot; , &#39;def&#39; | &quot;ghi&quot; ] ) [ &quot;abc&quot; , &#39;def&#39; | &quot;ghi&quot; ]","ref":"A.IO.html#to_iodata/1","title":"A.IO.to_iodata/1","type":"function"},{"doc":"Some extra helper functions for working with integers, that are not in the core Integer module.","ref":"A.Integer.html","title":"A.Integer","type":"module"},{"doc":"Format integers for humans, with thousand separators. Examples iex&gt; A.Integer . decimal_format ( 1_234_567 ) &quot;1,234,567&quot; iex&gt; A.Integer . decimal_format ( - 123 ) &quot;-123&quot; iex&gt; A.Integer . decimal_format ( - 1_234 , separator : ?_ ) &quot;-1_234&quot;","ref":"A.Integer.html#decimal_format/2","title":"A.Integer.decimal_format/2","type":"function"},{"doc":"Returns both the result of div/2 and rem/2 at once. Examples iex&gt; A.Integer . div_rem ( 7 , 3 ) { 2 , 1 } iex&gt; A.Integer . div_rem ( - 99 , 2 ) { - 49 , - 1 } iex&gt; A.Integer . div_rem ( 100 , 0 ) ** (ArithmeticError) bad argument in arithmetic expression","ref":"A.Integer.html#div_rem/2","title":"A.Integer.div_rem/2","type":"function"},{"doc":"Some extra helper functions for working with lists, that are not in the core List module.","ref":"A.List.html","title":"A.List","type":"module"},{"doc":"Prepends an element to a list, equivalent of [elem | list] that can be used in a pipe. Examples iex&gt; [ 2 , 3 , 5 , 8 ] |&gt; A.List . prepend ( 1 ) [ 1 , 2 , 3 , 5 , 8 ]","ref":"A.List.html#prepend/2","title":"A.List.prepend/2","type":"function"},{"doc":"Populates a list of size n by calling generator_fun repeatedly. Examples # Although not necessary, let&#39;s seed the random algorithm iex&gt; :rand . seed ( :exrop , { 1 , 2 , 3 } ) iex&gt; A.List . repeat ( &amp; :rand . uniform / 0 , 3 ) [ 0.7498295129076106 , 0.06161655489244533 , 0.7924073127680873 ] # It is basically just syntactic sugar for the following: iex&gt; Stream . repeatedly ( &amp; :rand . uniform / 0 ) |&gt; Enum . take ( 3 ) Rationale It offers has a consistent API with Stream.repeatedly/1 and List.duplicate/2 It provides a less verbose way of writing one of the most common uses of Stream.repeatedly/1 (before Elixir 1.12) It removes the temptation to write the following, which is more concise but is technically incorrect: iex&gt; incorrect = fn n -&gt; for _i &lt;- 1 .. n , do : :rand . uniform ( ) end iex&gt; incorrect . ( 0 ) |&gt; length ( ) 2 iex&gt; Enum . to_list ( 1 .. 0 ) # &lt;- because of this [ 1 , 0 ] Elixir 1.12 solved this problem by introducing a step, 1..n//1 . It is more efficient","ref":"A.List.html#repeat/2","title":"A.List.repeat/2","type":"function"},{"doc":"A map preserving key insertion order, with efficient lookups, updates and enumeration. It works like regular maps, except that the insertion order is preserved: iex&gt; %{ &quot;one&quot; =&gt; 1 , &quot;two&quot; =&gt; 2 , &quot;three&quot; =&gt; 3 } %{ &quot;one&quot; =&gt; 1 , &quot;three&quot; =&gt; 3 , &quot;two&quot; =&gt; 2 } iex&gt; A.OrdMap . new ( [ { &quot;one&quot; , 1 } , { &quot;two&quot; , 2 } , { &quot;three&quot; , 3 } ] ) ord ( %{ &quot;one&quot; =&gt; 1 , &quot;two&quot; =&gt; 2 , &quot;three&quot; =&gt; 3 } ) There is an unavoidable overhead compared to natively implemented maps, so keep using regular maps when you do not care about the insertion order. A.OrdMap : provides efficient (logarithmic) access: it is not a simple list of tuples implements the Access behaviour, Enum / Inspect / Collectable protocols optionally implements the Jason.Encoder protocol if Jason is installed Examples A.OrdMap offers the same API as Map : iex&gt; ord_map = A.OrdMap . new ( [ b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; ] ) ord ( %{ b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . get ( ord_map , :c ) &quot;Cat&quot; iex&gt; A.OrdMap . fetch ( ord_map , :a ) { :ok , &quot;Ant&quot; } iex&gt; A.OrdMap . put ( ord_map , :d , &quot;Dinosaur&quot; ) ord ( %{ b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; , d : &quot;Dinosaur&quot; } ) iex&gt; A.OrdMap . put ( ord_map , :b , &quot;Buffalo&quot; ) ord ( %{ b : &quot;Buffalo&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; } ) iex&gt; Enum . to_list ( ord_map ) [ b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; ] iex&gt; [ d : &quot;Dinosaur&quot; , b : &quot;Buffalo&quot; , e : &quot;Eel&quot; ] |&gt; Enum . into ( ord_map ) ord ( %{ b : &quot;Buffalo&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; , d : &quot;Dinosaur&quot; , e : &quot;Eel&quot; } ) Specific functions Due to its ordered nature, A.OrdMap also offers some extra methods not present in Map , like: first/1 and last/1 to efficiently retrieve the first / last key-value pair foldl/3 and foldr/3 to efficiently fold (reduce) from left-to-right or right-to-left Examples: iex&gt; ord_map = A.OrdMap . new ( b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . first ( ord_map ) { :b , &quot;Bat&quot; } iex&gt; A.OrdMap . last ( ord_map ) { :c , &quot;Cat&quot; } iex&gt; A.OrdMap . foldr ( ord_map , [ ] , fn { _key , value } , acc -&gt; [ value &lt;&gt; &quot;man&quot; | acc ] end ) [ &quot;Batman&quot; , &quot;Antman&quot; , &quot;Catman&quot; ] Access behaviour A.OrdMap implements the Access behaviour. iex&gt; ord_map = A.OrdMap . new ( [ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ] ) iex&gt; ord_map [ :a ] &quot;Ant&quot; iex&gt; put_in ( ord_map [ :b ] , &quot;Buffalo&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Buffalo&quot; , c : &quot;Cat&quot; } ) iex&gt; put_in ( ord_map [ :d ] , &quot;Dinosaur&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; , d : &quot;Dinosaur&quot; } ) iex&gt; { &quot;Cat&quot; , updated } = pop_in ( ord_map [ :c ] ) ; updated ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; } ) Convenience ord/1 and ord_size/1 macros The A.OrdMap module can be used without any macro. The A.ord/1 macro does however provide some syntactic sugar to make it more convenient to work with ordered maps, namely: construct new ordered maps without the clutter of a entry list pattern match on key-values like regular maps update some existing keys Examples: iex&gt; import A iex&gt; ord_map = ord ( %{ &quot;一&quot; =&gt; 1 , &quot;二&quot; =&gt; 2 , &quot;三&quot; =&gt; 3 } ) ord ( %{ &quot;一&quot; =&gt; 1 , &quot;二&quot; =&gt; 2 , &quot;三&quot; =&gt; 3 } ) iex&gt; ord ( %{ &quot;三&quot; =&gt; three , &quot;一&quot; =&gt; one } ) = ord_map iex&gt; { one , three } { 1 , 3 } iex&gt; ord ( %{ ord_map | &quot;二&quot; =&gt; &quot;NI!&quot; } ) ord ( %{ &quot;一&quot; =&gt; 1 , &quot;二&quot; =&gt; &quot;NI!&quot; , &quot;三&quot; =&gt; 3 } ) Notes: pattern-matching on keys is not affected by insertion order. For expressions with constant keys, A.ord/1 is able to generate the AST at compile time like the A.vec/1 macro. The A.ord_size/1 macro can be used in guards: iex&gt; import A iex&gt; match? ( v when ord_size ( v ) &gt; 2 , ord %{ &quot;一&quot; =&gt; 1 , &quot;二&quot; =&gt; 2 , &quot;三&quot; =&gt; 3 } ) true With Jason iex&gt; A.OrdMap . new ( [ { &quot;un&quot; , 1 } , { &quot;deux&quot; , 2 } , { &quot;trois&quot; , 3 } ] ) |&gt; Jason . encode! ( ) &quot;{ \\&quot; un \\&quot; :1, \\&quot; deux \\&quot; :2, \\&quot; trois \\&quot; :3}&quot; JSON encoding preserves the insertion order. Comparing with a regular map: iex&gt; Map . new ( [ { &quot;un&quot; , 1 } , { &quot;deux&quot; , 2 } , { &quot;trois&quot; , 3 } ] ) |&gt; Jason . encode! ( ) &quot;{ \\&quot; deux \\&quot; :2, \\&quot; trois \\&quot; :3, \\&quot; un \\&quot; :1}&quot; There is no way as of now to decode JSON using A.OrdMap . Key deletion and sparse maps Due to the underlying structures being used, efficient key deletion implies keeping around some &quot;holes&quot; to avoid rebuilding the whole structure. Such an ord map will be called sparse , while an ord map that never had a key deleted will be referred as dense . The implications of sparse structures are multiple: unlike dense structures, they cannot be compared as erlang terms (using either ==/2 , ===/2 or the pin operator ^ ) A.OrdMap.equal?/2 can safely compare both sparse and dense structures, but is slower for sparse enumerating sparse structures is less efficient than dense ones Calling A.OrdMap.new/1 on a sparse ord map will rebuild a new dense one from scratch (which can be expensive). iex&gt; dense = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; } ) iex&gt; sparse = A.OrdMap . new ( c : &quot;Cat&quot; , a : &quot;Ant&quot; , b : &quot;Bat&quot; ) |&gt; A.OrdMap . delete ( :c ) # A.OrdMap &lt; %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; } , sparse? : true &gt; iex&gt; dense == sparse false iex&gt; match? ( ^ dense , sparse ) false iex&gt; A.OrdMap . equal? ( dense , sparse ) # works with sparse maps, but less efficient true iex&gt; new_dense = A.OrdMap . new ( sparse ) # rebuild a dense map from a sparse one ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; } ) iex&gt; new_dense === dense true In order to avoid having to worry about memory issues when adding and deleting keys successively, ord maps cannot be more than half sparse, and are periodically rebuilt as dense upon deletion. iex&gt; sparse = A.OrdMap . new ( c : &quot;Cat&quot; , a : &quot;Ant&quot; , b : &quot;Bat&quot; ) |&gt; A.OrdMap . delete ( :c ) # A.OrdMap &lt; %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; } , sparse? : true &gt; iex&gt; A.OrdMap . delete ( sparse , :a ) ord ( %{ b : &quot;Bat&quot; } ) Note: Deleting the last key does not make a dense ord map sparse. This is not a bug, but an expected behavior due to how data is stored. iex&gt; A.OrdMap . new ( [ one : 1 , two : 2 , three : 3 ] ) |&gt; A.OrdMap . delete ( :three ) ord ( %{ one : 1 , two : 2 } ) The dense?/1 and sparse?/1 functions can be used to check if a A.OrdMap is dense or sparse. While this design puts some burden on the developer, the idea behind it is: to keep it as convenient and performant as possible unless deletion is necessary to be transparent about sparse structures and their limitation instead of constantly rebuild new dense structures, let users decide the best timing to do it still work fine with sparse structures, but in a degraded mode protect users about potential memory leaks and performance issues Pattern-matching and opaque type An A.OrdMap is represented internally using the %A.OrdMap{} struct. This struct can be used whenever there's a need to pattern match on something being an A.OrdMap : iex&gt; match? ( % A.OrdMap { } , A.OrdMap . new ( ) ) true Note, however, than A.OrdMap is an opaque type : its struct internal fields must not be accessed directly. As discussed in the previous section, ord/1 and ord_size/1 makes it possible to pattern match on keys as well as check the type and size. Memory overhead A.OrdMap takes roughly 2~3x more memory than a regular map depending on the type of data: iex&gt; map_size = Map . new ( 1 .. 100 , fn i -&gt; { i , i } end ) |&gt; :erts_debug . size ( ) 366 iex&gt; ord_map_size = A.OrdMap . new ( 1 .. 100 , fn i -&gt; { i , i } end ) |&gt; :erts_debug . size ( ) 1019 iex&gt; Float . round ( ord_map_size / map_size , 2 ) 2.78","ref":"A.OrdMap.html","title":"A.OrdMap","type":"module"},{"doc":"Deletes the entry in ord_map for a specific key . If the key does not exist, returns ord_map unchanged. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . delete ( ord_map , :b ) # A.OrdMap &lt; %{ a : &quot;Ant&quot; , c : &quot;Cat&quot; } , sparse? : true &gt; iex&gt; A.OrdMap . delete ( ord_map , :z ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } )","ref":"A.OrdMap.html#delete/2","title":"A.OrdMap.delete/2","type":"function"},{"doc":"Returns true if ord_map is dense; otherwise returns false . See the section about sparse structures for more information. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . dense? ( ord_map ) true iex&gt; sparse = A.OrdMap . delete ( ord_map , :b ) # A.OrdMap &lt; %{ a : &quot;Ant&quot; , c : &quot;Cat&quot; } , sparse? : true &gt; iex&gt; A.OrdMap . dense? ( sparse ) false","ref":"A.OrdMap.html#dense?/1","title":"A.OrdMap.dense?/1","type":"function"},{"doc":"Drops the given keys from ord_map . If keys contains keys that are not in ord_map , they're simply ignored. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . drop ( ord_map , [ :b , :d ] ) # A.OrdMap &lt; %{ a : &quot;Ant&quot; , c : &quot;Cat&quot; } , sparse? : true &gt;","ref":"A.OrdMap.html#drop/2","title":"A.OrdMap.drop/2","type":"function"},{"doc":"Checks if two ordered maps are equal, meaning they have the same key-value pairs in the same order. Examples iex&gt; A.OrdMap . equal? ( A.OrdMap . new ( a : 1 , b : 2 ) , A.OrdMap . new ( a : 1 , b : 2 ) ) true iex&gt; A.OrdMap . equal? ( A.OrdMap . new ( a : 1 , b : 2 ) , A.OrdMap . new ( b : 2 , a : 1 ) ) false iex&gt; A.OrdMap . equal? ( A.OrdMap . new ( a : 1 , b : 2 ) , A.OrdMap . new ( a : 3 , b : 2 ) ) false","ref":"A.OrdMap.html#equal?/2","title":"A.OrdMap.equal?/2","type":"function"},{"doc":"Fetches the value for a specific key and returns it in a ok-entry. If the key does not exist, returns :error. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;A&quot; , b : &quot;B&quot; , c : &quot;C&quot; ) iex&gt; A.OrdMap . fetch ( ord_map , :c ) { :ok , &quot;C&quot; } iex&gt; A.OrdMap . fetch ( ord_map , :z ) :error","ref":"A.OrdMap.html#fetch/2","title":"A.OrdMap.fetch/2","type":"function"},{"doc":"Fetches the value for a specific key in the given ord_map , erroring out if ord_map doesn't contain key . If ord_map doesn't contain key , a KeyError exception is raised. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;A&quot; , b : &quot;B&quot; , c : &quot;C&quot; ) iex&gt; A.OrdMap . fetch! ( ord_map , :c ) &quot;C&quot; iex&gt; A.OrdMap . fetch! ( ord_map , :z ) ** (KeyError) key :z not found in: ord(%{a: &quot;A&quot;, b: &quot;B&quot;, c: &quot;C&quot;})","ref":"A.OrdMap.html#fetch!/2","title":"A.OrdMap.fetch!/2","type":"function"},{"doc":"Finds the fist {key, value} pair in ord_map . Returns a {key, value} entry if ord_map is non-empty, or nil else. Examples iex&gt; A.OrdMap . new ( [ b : &quot;B&quot; , d : &quot;D&quot; , a : &quot;A&quot; , c : &quot;C&quot; ] ) |&gt; A.OrdMap . first ( ) { :b , &quot;B&quot; } iex&gt; A.OrdMap . new ( [ ] ) |&gt; A.OrdMap . first ( ) nil iex&gt; A.OrdMap . new ( [ ] ) |&gt; A.OrdMap . first ( :error ) :error","ref":"A.OrdMap.html#first/2","title":"A.OrdMap.first/2","type":"function"},{"doc":"Folds (reduces) the given ord_map from the left with the function fun . Requires an accumulator acc . Examples iex&gt; ord_map = A.OrdMap . new ( [ b : &quot;Bat&quot; , c : &quot;Cat&quot; , a : &quot;Ant&quot; ] ) iex&gt; A.OrdMap . foldl ( ord_map , &quot;&quot; , fn { _key , value } , acc -&gt; value &lt;&gt; acc end ) &quot;AntCatBat&quot; iex&gt; A.OrdMap . foldl ( ord_map , [ ] , fn { key , value } , acc -&gt; [ { key , value &lt;&gt; &quot;man&quot; } | acc ] end ) [ a : &quot;Antman&quot; , c : &quot;Catman&quot; , b : &quot;Batman&quot; ]","ref":"A.OrdMap.html#foldl/3","title":"A.OrdMap.foldl/3","type":"function"},{"doc":"Folds (reduces) the given ord_map from the right with the function fun . Requires an accumulator acc . Unlike linked lists, this is as efficient as foldl/3 . This can typically save a call to Enum.reverse/1 on the result when building a list. Examples iex&gt; ord_map = A.OrdMap . new ( [ b : &quot;Bat&quot; , c : &quot;Cat&quot; , a : &quot;Ant&quot; ] ) iex&gt; A.OrdMap . foldr ( ord_map , &quot;&quot; , fn { _key , value } , acc -&gt; value &lt;&gt; acc end ) &quot;BatCatAnt&quot; iex&gt; A.OrdMap . foldr ( ord_map , [ ] , fn { key , value } , acc -&gt; [ { key , value &lt;&gt; &quot;man&quot; } | acc ] end ) [ b : &quot;Batman&quot; , c : &quot;Catman&quot; , a : &quot;Antman&quot; ]","ref":"A.OrdMap.html#foldr/3","title":"A.OrdMap.foldr/3","type":"function"},{"doc":"Converts a struct to an ordered map. It accepts the struct module or a struct itself and simply removes the __struct__ field from the given struct or from a new struct generated from the given module. Example defmodule User do defstruct [ :name , :age ] end A.OrdMap . from_struct ( User ) ord ( %{ age : nil , name : nil } ) A.OrdMap . from_struct ( % User { name : &quot;john&quot; , age : 44 } ) ord ( %{ age : 44 , name : &quot;john&quot; } )","ref":"A.OrdMap.html#from_struct/1","title":"A.OrdMap.from_struct/1","type":"function"},{"doc":"Gets the value for a specific key in ord_map . If key is present in ord_map then its value value is returned. Otherwise, default is returned. If default is not provided, nil is used. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . get ( ord_map , :a ) &quot;Ant&quot; iex&gt; A.OrdMap . get ( ord_map , :z ) nil iex&gt; A.OrdMap . get ( ord_map , :z , &quot;Zebra&quot; ) &quot;Zebra&quot;","ref":"A.OrdMap.html#get/3","title":"A.OrdMap.get/3","type":"function"},{"doc":"Gets the value from key and updates it, all in one pass. Mirrors Map.get_and_update/3 , see its documentation. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; { &quot;bat&quot; , updated } = A.OrdMap . get_and_update ( ord_map , :b , fn current_value -&gt; ...&gt; { current_value &amp;&amp; String . downcase ( current_value ) , &quot;Buffalo&quot; } ...&gt; end ) iex&gt; updated ord ( %{ a : &quot;Ant&quot; , b : &quot;Buffalo&quot; , c : &quot;Cat&quot; } ) iex&gt; { nil , updated } = A.OrdMap . get_and_update ( ord_map , :z , fn current_value -&gt; ...&gt; { current_value &amp;&amp; String . downcase ( current_value ) , &quot;Zebra&quot; } ...&gt; end ) iex&gt; updated ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; , z : &quot;Zebra&quot; } ) iex&gt; { &quot;Bat&quot; , updated } = A.OrdMap . get_and_update ( ord_map , :b , fn _ -&gt; :pop end ) iex&gt; updated # A.OrdMap &lt; %{ a : &quot;Ant&quot; , c : &quot;Cat&quot; } , sparse? : true &gt; iex&gt; { nil , updated } = A.OrdMap . get_and_update ( ord_map , :z , fn _ -&gt; :pop end ) iex&gt; updated ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } )","ref":"A.OrdMap.html#get_and_update/3","title":"A.OrdMap.get_and_update/3","type":"function"},{"doc":"Gets the value from key and updates it, all in one pass. Mirrors Map.get_and_update!/3 , see its documentation. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; { &quot;bat&quot; , updated } = A.OrdMap . get_and_update! ( ord_map , :b , fn current_value -&gt; ...&gt; { current_value &amp;&amp; String . downcase ( current_value ) , &quot;Buffalo&quot; } ...&gt; end ) iex&gt; updated ord ( %{ a : &quot;Ant&quot; , b : &quot;Buffalo&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . get_and_update! ( ord_map , :z , fn current_value -&gt; ...&gt; { current_value &amp;&amp; String . downcase ( current_value ) , &quot;Zebra&quot; } ...&gt; end ) ** (KeyError) key :z not found in: ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})","ref":"A.OrdMap.html#get_and_update!/3","title":"A.OrdMap.get_and_update!/3","type":"function"},{"doc":"Gets the value for a specific key in ord_map . If key is present in ord_map then its value value is returned. Otherwise, fun is evaluated and its result is returned. This is useful if the default value is very expensive to calculate or generally difficult to setup and teardown again. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; expensive_fun = fn -&gt; &quot;Zebra&quot; end iex&gt; A.OrdMap . get_lazy ( ord_map , :a , expensive_fun ) &quot;Ant&quot; iex&gt; A.OrdMap . get_lazy ( ord_map , :z , expensive_fun ) &quot;Zebra&quot;","ref":"A.OrdMap.html#get_lazy/3","title":"A.OrdMap.get_lazy/3","type":"function"},{"doc":"Returns whether the given key exists in ord_map . Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . has_key? ( ord_map , :a ) true iex&gt; A.OrdMap . has_key? ( ord_map , :d ) false","ref":"A.OrdMap.html#has_key?/2","title":"A.OrdMap.has_key?/2","type":"function"},{"doc":"Returns all keys from ord_map . Examples iex&gt; ord_map = A.OrdMap . new ( b : &quot;Bat&quot; , c : &quot;Cat&quot; , a : &quot;Ant&quot; ) iex&gt; A.OrdMap . keys ( ord_map ) [ :b , :c , :a ]","ref":"A.OrdMap.html#keys/1","title":"A.OrdMap.keys/1","type":"function"},{"doc":"Finds the last {key, value} pair in ord_map . Returns a {key, value} entry if ord_map is non-empty, or nil else. Can be accessed efficiently due to the underlying vector. Examples iex&gt; A.OrdMap . new ( [ b : &quot;B&quot; , d : &quot;D&quot; , a : &quot;A&quot; , c : &quot;C&quot; ] ) |&gt; A.OrdMap . last ( ) { :c , &quot;C&quot; } iex&gt; A.OrdMap . new ( [ ] ) |&gt; A.OrdMap . last ( ) nil iex&gt; A.OrdMap . new ( [ ] ) |&gt; A.OrdMap . last ( :error ) :error","ref":"A.OrdMap.html#last/2","title":"A.OrdMap.last/2","type":"function"},{"doc":"Merges a map or an ordered map into an ord_map . All keys in map_or_ord_map will be added to ord_map , overriding any existing one (i.e., the keys in map_or_ord_map &quot;have precedence&quot; over the ones in ord_map ). Examples iex&gt; A.OrdMap . merge ( A.OrdMap . new ( %{ a : 1 , b : 2 } ) , A.OrdMap . new ( %{ a : 3 , d : 4 } ) ) ord ( %{ a : 3 , b : 2 , d : 4 } ) iex&gt; A.OrdMap . merge ( A.OrdMap . new ( %{ a : 1 , b : 2 } ) , %{ a : 3 , d : 4 } ) ord ( %{ a : 3 , b : 2 , d : 4 } )","ref":"A.OrdMap.html#merge/2","title":"A.OrdMap.merge/2","type":"function"},{"doc":"Returns a new empty ordered map. Examples iex&gt; A.OrdMap . new ( ) ord ( %{ } )","ref":"A.OrdMap.html#new/0","title":"A.OrdMap.new/0","type":"function"},{"doc":"Creates an ordered map from an enumerable . Preserves the original order of keys. Duplicated keys are removed; the latest one prevails. Examples iex&gt; A.OrdMap . new ( b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; ) ord ( %{ b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . new ( b : &quot;Bat&quot; , a : &quot;Ant&quot; , b : &quot;Buffalo&quot; , a : &quot;Antelope&quot; ) ord ( %{ b : &quot;Buffalo&quot; , a : &quot;Antelope&quot; } ) new/1 will return dense ord maps untouched, but will rebuild sparse ord maps from scratch. This can be used to build a dense ord map from from a sparse one. See the section about sparse structures for more information. iex&gt; sparse = A.OrdMap . new ( c : &quot;Cat&quot; , a : &quot;Ant&quot; , b : &quot;Bat&quot; ) |&gt; A.OrdMap . delete ( :c ) # A.OrdMap &lt; %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; } , sparse? : true &gt; iex&gt; A.OrdMap . new ( sparse ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; } )","ref":"A.OrdMap.html#new/1","title":"A.OrdMap.new/1","type":"function"},{"doc":"Creates an ordered map from an enumerable via the given transform function. Preserves the original order of keys. Duplicated keys are removed; the latest one prevails. Examples iex&gt; A.OrdMap . new ( [ :a , :b ] , fn x -&gt; { x , x } end ) ord ( %{ a : :a , b : :b } )","ref":"A.OrdMap.html#new/2","title":"A.OrdMap.new/2","type":"function"},{"doc":"Returns the value for key and the updated ordered map without key . If key is present in the ordered map with a value value , {value, new_ord_map} is returned. If key is not present in the ordered map, {default, ord_map} is returned. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; { &quot;Bat&quot; , updated } = A.OrdMap . pop ( ord_map , :b ) iex&gt; updated # A.OrdMap &lt; %{ a : &quot;Ant&quot; , c : &quot;Cat&quot; } , sparse? : true &gt; iex&gt; { nil , updated } = A.OrdMap . pop ( ord_map , :z ) iex&gt; updated ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } ) iex&gt; { &quot;Z&quot; , updated } = A.OrdMap . pop ( ord_map , :z , &quot;Z&quot; ) iex&gt; updated ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } )","ref":"A.OrdMap.html#pop/3","title":"A.OrdMap.pop/3","type":"function"},{"doc":"Returns the value for key and the updated ordered map without key . Behaves the same as pop/3 but raises if key is not present in ord_map . Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; { &quot;Bat&quot; , updated } = A.OrdMap . pop! ( ord_map , :b ) iex&gt; updated # A.OrdMap &lt; %{ a : &quot;Ant&quot; , c : &quot;Cat&quot; } , sparse? : true &gt; iex&gt; A.OrdMap . pop! ( ord_map , :z ) ** (KeyError) key :z not found in: ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})","ref":"A.OrdMap.html#pop!/2","title":"A.OrdMap.pop!/2","type":"function"},{"doc":"Lazily returns and removes the value associated with key in ord_map . If key is present in ord_map , it returns {value, new_map} where value is the value of the key and new_map is the result of removing key from ord_map . If key is not present in ord_map , {fun_result, ord_map} is returned, where fun_result is the result of applying fun . This is useful if the default value is very expensive to calculate or generally difficult to setup and teardown again. Examples iex&gt; ord_map = A.OrdMap . new ( b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; ) iex&gt; expensive_fun = fn -&gt; &quot;Zebra&quot; end iex&gt; { &quot;Ant&quot; , updated } = A.OrdMap . pop_lazy ( ord_map , :a , expensive_fun ) iex&gt; updated # A.OrdMap &lt; %{ b : &quot;Bat&quot; , c : &quot;Cat&quot; } , sparse? : true &gt; iex&gt; { &quot;Zebra&quot; , not_updated } = A.OrdMap . pop_lazy ( ord_map , :z , expensive_fun ) iex&gt; not_updated ord ( %{ b : &quot;Bat&quot; , a : &quot;Ant&quot; , c : &quot;Cat&quot; } )","ref":"A.OrdMap.html#pop_lazy/3","title":"A.OrdMap.pop_lazy/3","type":"function"},{"doc":"Puts the given value under key in ord_map . If the key does exist, it overwrites the existing value without changing its current location. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . put ( ord_map , :b , &quot;Buffalo&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Buffalo&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . put ( ord_map , :d , &quot;Dinosaur&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; , d : &quot;Dinosaur&quot; } )","ref":"A.OrdMap.html#put/3","title":"A.OrdMap.put/3","type":"function"},{"doc":"Puts the given value under key unless the entry key already exists in ord_map . Examples iex&gt; ord_map = A.OrdMap . new ( b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . put_new ( ord_map , :a , &quot;Ant&quot; ) ord ( %{ b : &quot;Bat&quot; , c : &quot;Cat&quot; , a : &quot;Ant&quot; } ) iex&gt; A.OrdMap . put_new ( ord_map , :b , &quot;Buffalo&quot; ) ord ( %{ b : &quot;Bat&quot; , c : &quot;Cat&quot; } )","ref":"A.OrdMap.html#put_new/3","title":"A.OrdMap.put_new/3","type":"function"},{"doc":"Evaluates fun and puts the result under key in ord_map unless key is already present. This function is useful in case you want to compute the value to put under key only if key is not already present, as for example, when the value is expensive to calculate or generally difficult to setup and teardown again. Examples iex&gt; ord_map = A.OrdMap . new ( b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; expensive_fun = fn -&gt; &quot;Ant&quot; end iex&gt; A.OrdMap . put_new_lazy ( ord_map , :a , expensive_fun ) ord ( %{ b : &quot;Bat&quot; , c : &quot;Cat&quot; , a : &quot;Ant&quot; } ) iex&gt; A.OrdMap . put_new_lazy ( ord_map , :b , expensive_fun ) ord ( %{ b : &quot;Bat&quot; , c : &quot;Cat&quot; } )","ref":"A.OrdMap.html#put_new_lazy/3","title":"A.OrdMap.put_new_lazy/3","type":"function"},{"doc":"Puts a value under key only if the key already exists in ord_map . Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . replace ( ord_map , :b , &quot;Buffalo&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Buffalo&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . replace ( ord_map , :d , &quot;Dinosaur&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } )","ref":"A.OrdMap.html#replace/3","title":"A.OrdMap.replace/3","type":"function"},{"doc":"Puts a value under key only if the key already exists in ord_map . If key is not present in ord_map , a KeyError exception is raised. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . replace! ( ord_map , :b , &quot;Buffalo&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Buffalo&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . replace! ( ord_map , :d , &quot;Dinosaur&quot; ) ** (KeyError) key :d not found in: ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})","ref":"A.OrdMap.html#replace!/3","title":"A.OrdMap.replace!/3","type":"function"},{"doc":"Returns the number of keys in ord_map . Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . size ( ord_map ) 3 iex&gt; A.OrdMap . size ( A.OrdMap . new ( ) ) 0","ref":"A.OrdMap.html#size/1","title":"A.OrdMap.size/1","type":"function"},{"doc":"Returns true if ord_map is sparse; otherwise returns false . See the section about sparse structures for more information. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . sparse? ( ord_map ) false iex&gt; sparse = A.OrdMap . delete ( ord_map , :b ) # A.OrdMap &lt; %{ a : &quot;Ant&quot; , c : &quot;Cat&quot; } , sparse? : true &gt; iex&gt; A.OrdMap . sparse? ( sparse ) true","ref":"A.OrdMap.html#sparse?/1","title":"A.OrdMap.sparse?/1","type":"function"},{"doc":"Returns a new ordered map with all the key-value pairs in ord_map where the key is in keys . If keys contains keys that are not in ord_map , they're simply ignored. Respects the order of the keys list. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . take ( ord_map , [ :c , :e , :a ] ) ord ( %{ c : &quot;Cat&quot; , a : &quot;Ant&quot; } )","ref":"A.OrdMap.html#take/2","title":"A.OrdMap.take/2","type":"function"},{"doc":"Returns all key-values pairs from ord_map as a list. Examples iex&gt; ord_map = A.OrdMap . new ( b : &quot;Bat&quot; , c : &quot;Cat&quot; , a : &quot;Ant&quot; ) iex&gt; A.OrdMap . to_list ( ord_map ) [ b : &quot;Bat&quot; , c : &quot;Cat&quot; , a : &quot;Ant&quot; ]","ref":"A.OrdMap.html#to_list/1","title":"A.OrdMap.to_list/1","type":"function"},{"doc":"Puts a value under key only if the key already exists in ord_map . Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . update ( ord_map , :b , &quot;N/A&quot; , &amp; String . upcase / 1 ) ord ( %{ a : &quot;Ant&quot; , b : &quot;BAT&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . update ( ord_map , :z , &quot;N/A&quot; , &amp; String . upcase / 1 ) ord ( %{ a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; , z : &quot;N/A&quot; } )","ref":"A.OrdMap.html#update/4","title":"A.OrdMap.update/4","type":"function"},{"doc":"Puts a value under key only if the key already exists in ord_map . If key is not present in ord_map , a KeyError exception is raised. Examples iex&gt; ord_map = A.OrdMap . new ( a : &quot;Ant&quot; , b : &quot;Bat&quot; , c : &quot;Cat&quot; ) iex&gt; A.OrdMap . update! ( ord_map , :b , &amp; String . upcase / 1 ) ord ( %{ a : &quot;Ant&quot; , b : &quot;BAT&quot; , c : &quot;Cat&quot; } ) iex&gt; A.OrdMap . update! ( ord_map , :d , &amp; String . upcase / 1 ) ** (KeyError) key :d not found in: ord(%{a: &quot;Ant&quot;, b: &quot;Bat&quot;, c: &quot;Cat&quot;})","ref":"A.OrdMap.html#update!/3","title":"A.OrdMap.update!/3","type":"function"},{"doc":"Returns all values from ord_map . Examples iex&gt; ord_map = A.OrdMap . new ( b : &quot;Bat&quot; , c : &quot;Cat&quot; , a : &quot;Ant&quot; ) iex&gt; A.OrdMap . values ( ord_map ) [ &quot;Bat&quot; , &quot;Cat&quot; , &quot;Ant&quot; ]","ref":"A.OrdMap.html#values/1","title":"A.OrdMap.values/1","type":"function"},{"doc":"","ref":"A.OrdMap.html#t:key/0","title":"A.OrdMap.key/0","type":"type"},{"doc":"","ref":"A.OrdMap.html#t:t/0","title":"A.OrdMap.t/0","type":"type"},{"doc":"","ref":"A.OrdMap.html#t:t/2","title":"A.OrdMap.t/2","type":"type"},{"doc":"","ref":"A.OrdMap.html#t:value/0","title":"A.OrdMap.value/0","type":"type"},{"doc":"Convenience helpers for working with {atom, value} tuples without breaking the pipe.","ref":"A.Pair.html","title":"A.Pair","type":"module"},{"doc":"Unwraps an idiomatic {atom, value} tuple when the atom is what is being expected. Convenient for not breaking the pipe. Examples iex&gt; { :ok , 55 } |&gt; A.Pair . unwrap! ( :ok ) 55 iex&gt; :error |&gt; A.Pair . unwrap! ( :ok ) ** (ArgumentError) unwrap!/2 expected {:ok, _}, got: :error","ref":"A.Pair.html#unwrap!/2","title":"A.Pair.unwrap!/2","type":"function"},{"doc":"Wraps the value as an idiomatic {atom, value} tuple. Convenient for not breaking the pipe. Examples iex&gt; 55 |&gt; A.Pair . wrap ( :ok ) { :ok , 55 } iex&gt; %{ a : 5 } |&gt; Map . update! ( :a , &amp; &amp;1 + 1 ) |&gt; A.Pair . wrap ( :no_reply ) { :no_reply , %{ a : 6 } }","ref":"A.Pair.html#wrap/2","title":"A.Pair.wrap/2","type":"function"},{"doc":"Some extra helper functions for working with strings, that are not in the core String module.","ref":"A.String.html","title":"A.String","type":"module"},{"doc":"Transforms the text as a slug. Removes whitespace, special characters and converts the rest to lowercase. This is typically useful to generate URLs based on content, e.g. the title of an article. Like String.downcase/2 , slugify/2 also can handle the following modes: :default (keeps unicode), :ascii or :greek . Examples iex&gt; A.String . slugify ( &quot;&gt; \\&quot; It Was Me, Dio!!! \\&quot; \\n &quot; ) &quot;it-was-me-dio&quot; iex&gt; A.String . slugify ( &quot;Joseph Joestar a.k.a ジョジョ&quot; ) &quot;joseph-joestar-aka-ジョジョ&quot; iex&gt; A.String . slugify ( &lt;&lt; 220 &gt;&gt; ) ** (ArgumentError) Invalid string &lt;&lt;220&gt;&gt; :ascii converts to ascii when possible or strips characters: iex&gt; A.String . slugify ( &quot;OLÁ! \\n &quot; , :ascii ) &quot;ola&quot; iex&gt; A.String . slugify ( &quot;DIOの世界 -さらば友よ- &quot; , :ascii ) &quot;dio&quot; :greek handles the context sensitive sigma in Greek: iex&gt; A.String . slugify ( &quot; \\t ΣΣ?&quot; ) &quot;σσ&quot; iex&gt; A.String . slugify ( &quot; \\t ΣΣ?&quot; , :greek ) &quot;σς&quot;","ref":"A.String.html#slugify/2","title":"A.String.slugify/2","type":"function"},{"doc":"","ref":"A.String.html#t:mode/0","title":"A.String.mode/0","type":"type"},{"doc":"Fast persistent vector with efficient appends and random access. Persistent vectors have been introduced by Clojure as an efficient alternative to lists. Many operations for A.Vector run in effective constant time (length, random access, appends...), unlike linked lists for which most operations run in linear time. Functions that need to go through the whole collection like map/2 or foldl/3 are as often fast as their list equivalents, or sometimes even slightly faster. Vectors also use less memory than lists for &quot;big&quot; collections (see the Memory usage section ). Make sure to read the Efficiency guide section to get the best performance out of vectors. Erlang's :array module offer similar functionalities. However A.Vector : is a better Elixir citizen: pipe-friendliness, Access behaviour, Enum / Inspect / Collectable protocols is heavily optimized and should offer higher performance in most use cases, especially &quot;loops&quot; like map/2 / to_list/1 / foldl/3 mirrors most of the Enum module API (together with A.Enum ) with highly optimized versions for vectors ( A.Enum.join/1 , A.Enum.sum/1 , A.Enum.random/1 ...) supports negative indexing (e.g. -1 corresponds to the last element) optionally implements the Jason.Encoder protocol if Jason is installed Note: most of the design is inspired by this series of blog posts describing the Clojure implementation, but a branching factor of 16 = 2 ^ 4 has been picked instead of 32 = 2 ^ 5 . This choice was made following performance benchmarking that showed better overall performance for this particular implementation. Examples iex&gt; vector = A.Vector . new ( 1 .. 10 ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 ] ) iex&gt; A.Vector . append ( vector , :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 , :foo ] ) iex&gt; vector [ 3 ] 4 iex&gt; A.Vector . replace_at ( vector , - 1 , :bar ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , :bar ] ) iex&gt; 3 in vector true Access behaviour A.Vector implements the Access behaviour. iex&gt; vector = A.Vector . new ( 1 .. 10 ) iex&gt; vector [ 3 ] 4 iex&gt; put_in ( vector [ 5 ] , :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , :foo , 7 , 8 , 9 , 10 ] ) iex&gt; { 9 , updated } = pop_in ( vector [ 8 ] ) ; updated vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 10 ] ) Convenience vec/1 and vec_size/1 macros The A.Vector module can be used without any macro. The A.vec/1 macro does however provide some syntactic sugar to make it more convenient to work with vectors of known size, namely: pattern match on elements for vectors of known size construct new vectors of known size faster, by generating the AST at compile time Examples: iex&gt; import A iex&gt; vec ( [ 1 , 2 , 3 ] ) vec ( [ 1 , 2 , 3 ] ) iex&gt; vec ( [ 1 , 2 , var , _ , _ , _ ] ) = A.Vector . new ( 1 .. 6 ) ; var 3 iex&gt; vec ( first ||| last ) = A.Vector . new ( 0 .. 99_999 ) ; { first , last } { 0 , 99999 } The A.vec_size/1 macro can be used in guards: iex&gt; import A iex&gt; match? ( v when vec_size ( v ) &gt; 99 , A.Vector . new ( 1 .. 100 ) ) true Convenience +++/2 operator The A.+++/2 operator can make appending to a vector more compact by aliasing A.Vector.concat/2 : iex&gt; import A iex&gt; vec ( [ 1 , 2 , 3 ] ) ++ + vec ( [ 4 , 5 ] ) vec ( [ 1 , 2 , 3 , 4 , 5 ] ) Pattern-matching and opaque type An A.Vector is represented internally using the %A.Vector{} struct. This struct can be used whenever there's a need to pattern match on something being an A.Vector : iex&gt; match? ( % A.Vector { } , A.Vector . new ( ) ) true Note, however, than A.Vector is an opaque type : its struct internal fields must not be accessed directly. As discussed in the previous section, vec/1 makes it possible to pattern match on size and elements as well as checking the type. Memory usage Vectors have a small overhead over lists for smaller collections, but are using far less memory for bigger collections: iex&gt; memory_for = fn n -&gt; [ Enum . to_list ( 1 .. n ) , A.Vector . new ( 1 .. n ) ] |&gt; Enum . map ( &amp; :erts_debug . size / 1 ) end iex&gt; memory_for . ( 1 ) [ 2 , 32 ] iex&gt; memory_for . ( 10 ) [ 20 , 32 ] iex&gt; memory_for . ( 100 ) [ 200 , 151 ] iex&gt; memory_for . ( 10_000 ) [ 20000 , 11371 ] If you need to work with vectors containing mostly the same value, A.Vector.duplicate/2 is highly efficient both in time and memory (logarithmic). It minimizes the number of actual copies and reuses the same nested structures under the hood: iex&gt; A.Vector . duplicate ( 0 , 10_000 ) |&gt; :erts_debug . size ( ) 117 iex&gt; A.Vector . duplicate ( 0 , 10_000 ) |&gt; :erts_debug . flat_size ( ) # when shared over processes / ETS 11371 Even a 1B x 1B matrix of the same element costs virtually nothing! big_n = 1_000_000_000 0 |&gt; A.Vector . duplicate ( big_n ) |&gt; A.Vector . duplicate ( big_n ) |&gt; :erts_debug . size ( ) 539 Efficiency guide If you are using vectors and not lists, chances are that you care about performance. Here are a couple notes about how to use vectors in an optimal way. Most functions from this module are highly efficient, those that are not will indicate it in their documentation. But remember the golden rule: in case of doubt, always benchmark . Avoid prepending Appending is very efficient, but prepending is highly inefficient since the whole array needs to be reconstructed. DON'T A.Vector . prepend ( vector , :foo ) DO [ :foo | list ] # use lists A.Vector . append ( vector , :foo ) Avoid deletions This implementation of persistent vectors has many advantages, but it does not support efficient deletion, with the exception of the last element that can be popped very efficiently ( A.Vector.pop_last/1 , A.Vector.delete_last/1 ). Deleting close to the end of the vector using A.Vector.delete_at/2 or A.Vector.pop_at/3 is still fairly fast, but deleting near the beginning needs to reconstruct most of the vector. If you need to be able to delete arbitrary indexes, chances are you should consider an alternative data structure. Another possibility could be to use sparse arrays, defining nil as a deleted value (but then the indexing and size won't reflect this). DON'T A.Vector . pop_at ( vector , 3 ) A.Vector . delete_at ( vector , 3 ) pop_in ( vector [ 3 ] ) DO A.Vector . pop_last ( vector ) A.Vector . delete_last ( vector ) A.Vector . delete_at ( vector , - 3 ) # close to the end A.Vector . replace_at ( vector , 3 , nil ) Successive appends If you just need to append all elements of an enumerable, it is more efficient to use A.Vector.concat/2 or its alias A.+++/2 than successive calls to A.Vector.append/2 : DON'T Enum . reduce ( enumerable , vector , fn val , acc -&gt; A.Vector . append ( acc , val ) end ) DO A.Vector . concat ( vector , enumerable ) vector ++ + enumerable Prefer A.Enum and A.Vector to Enum for vectors The A.Enum module reimplements (nearly) all functions from the Enum module to offer optimal performance when operating on vectors, and should be used over Enum functions whenever possible (even if A.Vector implements the Enumerable and Collectable protocols for convienience): DON'T Enum . sum ( vector ) Enum . to_list ( vector ) Enum . reduce ( vector , [ ] , fun ) Enum . into ( enumerable , % A.Vector . new ( ) ) Enum . into ( enumerable , vector ) DO A.Enum . sum ( vector ) A.Enum . to_list ( vector ) # or A.Vector.to_list(vector) A.Enum . reduce ( vector , [ ] , fun ) # or A.Vector.foldl(vector, [], fun) A.Vector . new ( enumerable ) A.Enum . into ( enumerable , vector ) # or A.Vector.concat(vector, enumerable) # or vector +++ enumerable Although it depends on the function, you can expect a ~10x speed difference. for comprehensions are actually using Enumerable as well, so the same advice holds: DON'T for value &lt;- vector do do_stuff ( ) end If using it in EEx templates, you might want to cast it to a list: for value &lt;- A.Vector . to_list ( vector ) do do_stuff ( ) end Exceptions: Enum optimized functions Enum.member?/2 is implemented in an efficient way, so in/2 is optimal: DO 33 in vector Slicing optimization Slicing any subset on the left on the vector using methods from A.Vector is extremely efficient as the vector internals can be reused: DO A.Vector . take ( vector , 10 ) # take a positive amount A.Vector . drop ( vector , - 20 ) # drop a negative amount A.Vector . slice ( vector , 0 , 10 ) # slicing from 0 A.Vector . slice ( vector , 0 .. - 5 ) # slicing from 0 A.Vector and A.Enum A.Enum mirrors Enum and should return identical results, therefore many functions would return lists A.Vector mirrors Enum functions that are returning lists, but returns vectors instead iex&gt; vector = A.Vector.new(1..10) iex&gt; A.Enum.reverse(vector) [10, 9, 8, 7, 6, 5, 4, 3, 2, 1] iex&gt; A.Vector.reverse(vector) vec([10, 9, 8, 7, 6, 5, 4, 3, 2, 1]) iex&gt; A.Enum.map(vector, &amp; (&amp;1 7)) [7, 14, 21, 28, 35, 42, 49, 56, 63, 70] iex&gt; A.Vector.map(vector, &amp; (&amp;1 7)) vec([7, 14, 21, 28, 35, 42, 49, 56, 63, 70]) Additional notes If you need to work with vectors containing mostly the same value, use A.Vector.duplicate/2 (more details in the Memory usage section ). If you work with functions returning vectors of known size, you can use the A.vec/1 macro to defer the generation of the AST for the internal structure to compile time instead of runtime. A.Vector . new ( [ %{ foo : a } , %{ foo : b } ] ) # structure created at runtime vec ( [ %{ foo : a } , %{ foo : b } ] ) # structure AST defined at compile time","ref":"A.Vector.html","title":"A.Vector","type":"module"},{"doc":"Appends a value at the end of a vector . Runs in effective constant time. Examples iex&gt; A.Vector . new ( ) |&gt; A.Vector . append ( :foo ) vec ( [ :foo ] ) iex&gt; A.Vector . new ( 1 .. 5 ) |&gt; A.Vector . append ( :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , :foo ] )","ref":"A.Vector.html#append/2","title":"A.Vector.append/2","type":"function"},{"doc":"Finds the element at the given index (zero-based). Returns default if index is out of bounds. Supports negative indexing from the end of the vector . Runs in effective constant time. Examples iex&gt; A.Vector . new ( 1 .. 1_000 ) |&gt; A.Vector . at ( 555 ) 556 iex&gt; A.Vector . new ( 1 .. 1_000 ) |&gt; A.Vector . at ( 1_000 ) nil","ref":"A.Vector.html#at/2","title":"A.Vector.at/2","type":"function"},{"doc":"","ref":"A.Vector.html#at/3","title":"A.Vector.at/3","type":"function"},{"doc":"Finds the element at the given index (zero-based). Raises an A.Vector.IndexError if index is out of bounds. Supports negative indexing from the end of the vector . Runs in effective constant time. Examples iex&gt; A.Vector . new ( 1 .. 1_000 ) |&gt; A.Vector . at! ( 555 ) 556 iex&gt; A.Vector . new ( 1 .. 1_000 ) |&gt; A.Vector . at! ( - 10 ) 991 iex&gt; A.Vector . new ( 1 .. 1_000 ) |&gt; A.Vector . at! ( 1_000 ) ** (A.Vector.IndexError) out of bound index: 1000 not in -1000..999","ref":"A.Vector.html#at!/2","title":"A.Vector.at!/2","type":"function"},{"doc":"Appends all values from an enumerable at the end of a vector . Runs in effective linear time in respect with the length of enumerable , disregarding the size of the vector . Examples iex&gt; A.Vector . new ( 1 .. 5 ) |&gt; A.Vector . concat ( 10 .. 15 ) vec ( [ 1 , 2 , 3 , 4 , 5 , 10 , 11 , 12 , 13 , 14 , 15 ] ) iex&gt; A.Vector . new ( ) |&gt; A.Vector . concat ( 10 .. 15 ) vec ( [ 10 , 11 , 12 , 13 , 14 , 15 ] )","ref":"A.Vector.html#concat/2","title":"A.Vector.concat/2","type":"function"},{"doc":"Returns a copy of the vector where all consecutive duplicated elements are collapsed to a single element. Elements are compared using ===/2 . If you want to remove all duplicated elements, regardless of order, see uniq/1 . Examples iex&gt; A.Vector . new ( [ 1 , 2 , 3 , 3 , 2 , 1 ] ) |&gt; A.Vector . dedup ( ) vec ( [ 1 , 2 , 3 , 2 , 1 ] ) iex&gt; A.Vector . new ( [ 1 , 1 , 2 , 2.0 , :three , :three ] ) |&gt; A.Vector . dedup ( ) vec ( [ 1 , 2 , 2.0 , :three ] )","ref":"A.Vector.html#dedup/1","title":"A.Vector.dedup/1","type":"function"},{"doc":"Returns a copy of the vector where all consecutive duplicated elements are collapsed to a single element. The function fun maps every element to a term which is used to determine if two elements are duplicates. Examples iex&gt; vector = A.Vector . new ( [ { 1 , :a } , { 2 , :b } , { 2 , :c } , { 1 , :a } ] ) iex&gt; A.Vector . dedup_by ( vector , fn { x , _ } -&gt; x end ) vec ( [ { 1 , :a } , { 2 , :b } , { 1 , :a } ] ) iex&gt; vector = A.Vector . new ( [ 5 , 1 , 2 , 3 , 2 , 1 ] ) iex&gt; A.Vector . dedup_by ( vector , fn x -&gt; x &gt; 2 end ) vec ( [ 5 , 1 , 3 , 2 ] )","ref":"A.Vector.html#dedup_by/2","title":"A.Vector.dedup_by/2","type":"function"},{"doc":"(Inefficient) Returns a copy of vector without the value at the specified index . Returns the vector untouched if index is out of bounds. Supports negative indexing from the end of the vector . Runs in linear time. Its usage is discouraged, see the Efficiency guide . Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; A.Vector . delete_at ( vector , 4 ) vec ( [ 1 , 2 , 3 , 4 , 6 , 7 , 8 ] ) iex&gt; A.Vector . delete_at ( vector , - 9 ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ] )","ref":"A.Vector.html#delete_at/2","title":"A.Vector.delete_at/2","type":"function"},{"doc":"(Inefficient) Returns a copy of vector without the value at the specified index . Raises an A.Vector.IndexError if index is out of bounds. Supports negative indexing from the end of the vector . Runs in linear time. Its usage is discouraged, see the Efficiency guide . Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; A.Vector . delete_at! ( vector , 4 ) vec ( [ 1 , 2 , 3 , 4 , 6 , 7 , 8 ] ) iex&gt; A.Vector . delete_at! ( vector , - 9 ) ** (A.Vector.IndexError) out of bound index: -9 not in -8..7","ref":"A.Vector.html#delete_at!/2","title":"A.Vector.delete_at!/2","type":"function"},{"doc":"Removes the last value from the vector and returns the updated vector. Leaves the vector untouched if empty. Runs in effective constant time. Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; A.Vector . delete_last ( vector ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 ] ) iex&gt; A.Vector . delete_last ( A.Vector . new ( ) ) vec ( [ ] )","ref":"A.Vector.html#delete_last/1","title":"A.Vector.delete_last/1","type":"function"},{"doc":"Removes the last value from the vector and returns the updated vector. Raises an A.Vector.EmptyError if empty. Runs in effective constant time. Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; A.Vector . delete_last! ( vector ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 ] ) iex&gt; A.Vector . delete_last! ( A.Vector . new ( ) ) ** (A.Vector.EmptyError) empty vector error","ref":"A.Vector.html#delete_last!/1","title":"A.Vector.delete_last!/1","type":"function"},{"doc":"Drops the amount of elements from the vector . If a negative amount is given, the amount of last values will be dropped. Time complexity is: linear when amount is positive, as the vector needs to be reconstructed. effective constant time when amount is negative, as the vector structure can be shared Examples iex&gt; A.Vector . new ( 0 .. 15 ) |&gt; A.Vector . drop ( 10 ) vec ( [ 10 , 11 , 12 , 13 , 14 , 15 ] ) iex&gt; A.Vector . new ( 0 .. 5 ) |&gt; A.Vector . drop ( 0 ) vec ( [ 0 , 1 , 2 , 3 , 4 , 5 ] ) iex&gt; A.Vector . new ( 0 .. 10 ) |&gt; A.Vector . drop ( - 5 ) vec ( [ 0 , 1 , 2 , 3 , 4 , 5 ] )","ref":"A.Vector.html#drop/2","title":"A.Vector.drop/2","type":"function"},{"doc":"Drops elements at the beginning of the vector while fun returns a truthy value. Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . drop_while ( fn x -&gt; x &lt; 7 end ) vec ( [ 7 , 8 , 9 , 10 ] ) iex&gt; A.Vector . new ( [ 1 , true , %{ } , nil , &quot;abc&quot; ] ) |&gt; A.Vector . drop_while ( fn x -&gt; x end ) vec ( [ nil , &quot;abc&quot; ] )","ref":"A.Vector.html#drop_while/2","title":"A.Vector.drop_while/2","type":"function"},{"doc":"Duplicates the given element n times in a vector. n is an integer greater than or equal to 0 . If n is 0 , an empty list is returned. Runs in logarithmic time regarding n . It is very fast and memory efficient (see Memory usage ). Examples iex&gt; A.Vector . duplicate ( nil , 10 ) vec ( [ nil , nil , nil , nil , nil , nil , nil , nil , nil , nil ] ) iex&gt; A.Vector . duplicate ( :foo , 0 ) vec ( [ ] )","ref":"A.Vector.html#duplicate/2","title":"A.Vector.duplicate/2","type":"function"},{"doc":"Finds the element at the given index (zero-based), and returns it in a ok-entry. If the index does not exist, returns :error . Supports negative indexing from the end of the vector . Runs in effective constant time. Examples iex&gt; A.Vector . new ( 1 .. 1_000 ) |&gt; A.Vector . fetch ( 555 ) { :ok , 556 } iex&gt; A.Vector . new ( 1 .. 1_000 ) |&gt; A.Vector . fetch ( 1_000 ) :error iex&gt; A.Vector . new ( 1 .. 1_000 ) |&gt; A.Vector . fetch ( - 1 ) { :ok , 1000 }","ref":"A.Vector.html#fetch/2","title":"A.Vector.fetch/2","type":"function"},{"doc":"See A.Vector.at!/2 .","ref":"A.Vector.html#fetch!/2","title":"A.Vector.fetch!/2","type":"function"},{"doc":"Filters the vector , i.e. return a new vector containing only elements for which fun returns a truthy (neither false nor nil ) value. Runs in linear time. Examples iex&gt; vector = A.Vector . new ( 1 .. 100 ) iex&gt; A.Vector . filter ( vector , fn i -&gt; rem ( i , 13 ) == 0 end ) vec ( [ 13 , 26 , 39 , 52 , 65 , 78 , 91 ] )","ref":"A.Vector.html#filter/2","title":"A.Vector.filter/2","type":"function"},{"doc":"Returns the first element in the vector or default if vector is empty. Runs in actual constant time. Examples iex&gt; A.Vector . new ( 1 .. 10_000 ) |&gt; A.Vector . first ( ) 1 iex&gt; A.Vector . new ( ) |&gt; A.Vector . first ( ) nil","ref":"A.Vector.html#first/2","title":"A.Vector.first/2","type":"function"},{"doc":"Maps the given fun over vector and flattens the result. This function returns a new vector built by concatenating the results of invoking fun on each element of vector together. Runs in linear time. Examples iex&gt; A.Vector . new ( 0 .. 4 ) |&gt; A.Vector . flat_map ( fn i -&gt; List . duplicate ( i , i ) end ) vec ( [ 1 , 2 , 2 , 3 , 3 , 3 , 4 , 4 , 4 , 4 ] )","ref":"A.Vector.html#flat_map/2","title":"A.Vector.flat_map/2","type":"function"},{"doc":"Folds (reduces) the given vector from the left with the function fun . Requires an accumulator acc . Same as reduce/3 . Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . foldl ( 0 , &amp; + / 2 ) 55 iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . foldl ( [ ] , &amp; [ &amp;1 | &amp;2 ] ) [ 10 , 9 , 8 , 7 , 6 , 5 , 4 , 3 , 2 , 1 ]","ref":"A.Vector.html#foldl/3","title":"A.Vector.foldl/3","type":"function"},{"doc":"Folds (reduces) the given vector from the right with the function fun . Requires an accumulator acc . Unlike linked lists, this is as efficient as foldl/3 . This can typically save a call to Enum.reverse/1 on the result when building a list. Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . foldr ( 0 , &amp; + / 2 ) 55 iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . foldr ( [ ] , &amp; [ &amp;1 | &amp;2 ] ) [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 ]","ref":"A.Vector.html#foldr/3","title":"A.Vector.foldr/3","type":"function"},{"doc":"Gets the value from key and updates it, all in one pass. See Access.get_and_update/3 for more details. Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; { 6 , updated } = A.Vector . get_and_update ( vector , 5 , fn current_value -&gt; ...&gt; { current_value , current_value &amp;&amp; current_value * 100 } ...&gt; end ) ; updated vec ( [ 1 , 2 , 3 , 4 , 5 , 600 , 7 , 8 ] ) iex&gt; { nil , updated } = A.Vector . get_and_update ( vector , 8 , fn current_value -&gt; ...&gt; { current_value , current_value &amp;&amp; current_value * 100 } ...&gt; end ) ; updated vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ] ) iex&gt; { 4 , updated } = A.Vector . get_and_update ( vector , 3 , fn _ -&gt; :pop end ) ; updated vec ( [ 1 , 2 , 3 , 5 , 6 , 7 , 8 ] ) iex&gt; { nil , updated } = A.Vector . get_and_update ( vector , 8 , fn _ -&gt; :pop end ) ; updated vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ] )","ref":"A.Vector.html#get_and_update/3","title":"A.Vector.get_and_update/3","type":"function"},{"doc":"Intersperses separator between each element of the vector . Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 6 ) |&gt; A.Vector . intersperse ( nil ) vec ( [ 1 , nil , 2 , nil , 3 , nil , 4 , nil , 5 , nil , 6 ] )","ref":"A.Vector.html#intersperse/2","title":"A.Vector.intersperse/2","type":"function"},{"doc":"Returns the last element in the vector or default if vector is empty. Runs in constant time (actual, not effective). Examples iex&gt; A.Vector . new ( 1 .. 10_000 ) |&gt; A.Vector . last ( ) 10_000 iex&gt; A.Vector . new ( ) |&gt; A.Vector . last ( ) nil","ref":"A.Vector.html#last/2","title":"A.Vector.last/2","type":"function"},{"doc":"Returns a new vector where each element is the result of invoking fun on each corresponding element of vector . Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . map ( &amp; ( &amp;1 * &amp;1 ) ) vec ( [ 1 , 4 , 9 , 16 , 25 , 36 , 49 , 64 , 81 , 100 ] )","ref":"A.Vector.html#map/2","title":"A.Vector.map/2","type":"function"},{"doc":"Maps and intersperses the vector in one pass. Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 6 ) |&gt; A.Vector . map_intersperse ( nil , &amp; ( &amp;1 * 10 ) ) vec ( [ 10 , nil , 20 , nil , 30 , nil , 40 , nil , 50 , nil , 60 ] )","ref":"A.Vector.html#map_intersperse/3","title":"A.Vector.map_intersperse/3","type":"function"},{"doc":"Invokes the given fun to each element in the vector to reduce it to a single element, while keeping an accumulator. Returns a tuple where the first element is the mapped vector and the second one is the final accumulator. The function, fun , receives two arguments: the first one is the element, and the second one is the accumulator. fun must return a tuple with two elements in the form of {result, accumulator} . Examples iex&gt; vector = A.Vector . new ( [ 1 , 2 , 3 ] ) iex&gt; { new_vec , 6 } = A.Vector . map_reduce ( vector , 0 , fn x , acc -&gt; { x * 2 , x + acc } end ) iex&gt; new_vec vec ( [ 2 , 4 , 6 ] ) For example, if with_index/2 was not implemented, you could implement it as follows: iex&gt; vector = A.Vector . new ( [ 1 , 2 , 3 ] ) iex&gt; A.Vector . map_reduce ( vector , 0 , fn x , i -&gt; { { x , i } , i + 1 } end ) |&gt; elem ( 0 ) vec ( [ { 1 , 0 } , { 2 , 1 } , { 3 , 2 } ] )","ref":"A.Vector.html#map_reduce/3","title":"A.Vector.map_reduce/3","type":"function"},{"doc":"Returns a new empty vector. Examples iex&gt; A.Vector . new ( ) vec ( [ ] )","ref":"A.Vector.html#new/0","title":"A.Vector.new/0","type":"function"},{"doc":"Creates a vector from an enumerable . Runs in linear time. Examples iex&gt; A.Vector . new ( 10 .. 25 ) vec ( [ 10 , 11 , 12 , 13 , 14 , 15 , 16 , 17 , 18 , 19 , 20 , 21 , 22 , 23 , 24 , 25 ] )","ref":"A.Vector.html#new/1","title":"A.Vector.new/1","type":"function"},{"doc":"Creates a vector from an enumerable via the given transform function. Examples iex&gt; A.Vector . new ( 1 .. 10 , &amp; ( &amp;1 * &amp;1 ) ) vec ( [ 1 , 4 , 9 , 16 , 25 , 36 , 49 , 64 , 81 , 100 ] )","ref":"A.Vector.html#new/2","title":"A.Vector.new/2","type":"function"},{"doc":"(Inefficient) Returns and removes the value at the specified index in the vector . Returns the vector untouched if index is out of bounds. Supports negative indexing from the end of the vector . Runs in linear time. Its usage is discouraged, see the Efficiency guide . Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; { 5 , updated } = A.Vector . pop_at ( vector , 4 ) ; updated vec ( [ 1 , 2 , 3 , 4 , 6 , 7 , 8 ] ) iex&gt; { nil , updated } = A.Vector . pop_at ( vector , - 9 ) ; updated vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ] )","ref":"A.Vector.html#pop_at/3","title":"A.Vector.pop_at/3","type":"function"},{"doc":"(Inefficient) Returns and removes the value at the specified index in the vector . Raises an A.Vector.IndexError if index is out of bounds. Supports negative indexing from the end of the vector . Runs in linear time. Its usage is discouraged, see the Efficiency guide . Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; { 5 , updated } = A.Vector . pop_at! ( vector , 4 ) ; updated vec ( [ 1 , 2 , 3 , 4 , 6 , 7 , 8 ] ) iex&gt; A.Vector . pop_at! ( vector , - 9 ) ** (A.Vector.IndexError) out of bound index: -9 not in -8..7","ref":"A.Vector.html#pop_at!/2","title":"A.Vector.pop_at!/2","type":"function"},{"doc":"Removes the last value from the vector and returns both the value and the updated vector. Leaves the vector untouched if empty. Runs in effective constant time. Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; { 8 , updated } = A.Vector . pop_last ( vector ) ; updated vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 ] ) iex&gt; { nil , updated } = A.Vector . pop_last ( A.Vector . new ( ) ) ; updated vec ( [ ] )","ref":"A.Vector.html#pop_last/2","title":"A.Vector.pop_last/2","type":"function"},{"doc":"Removes the last value from the vector and returns both the value and the updated vector. Raises an A.Vector.EmptyError if empty. Runs in effective constant time. Examples iex&gt; vector = A.Vector . new ( 1 .. 8 ) iex&gt; { 8 , updated } = A.Vector . pop_last! ( vector ) ; updated vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 ] ) iex&gt; { nil , updated } = A.Vector . pop_last! ( A.Vector . new ( ) ) ; updated ** (A.Vector.EmptyError) empty vector error","ref":"A.Vector.html#pop_last!/1","title":"A.Vector.pop_last!/1","type":"function"},{"doc":"(Inefficient) Prepends value at the beginning of the vector . Runs in linear time because the whole vector needs to be reconstructuded, and should be avoided. Examples iex&gt; A.Vector . new ( ) |&gt; A.Vector . prepend ( :foo ) vec ( [ :foo ] ) iex&gt; A.Vector . new ( 1 .. 5 ) |&gt; A.Vector . prepend ( :foo ) vec ( [ :foo , 1 , 2 , 3 , 4 , 5 ] )","ref":"A.Vector.html#prepend/2","title":"A.Vector.prepend/2","type":"function"},{"doc":"Filters the vector , i.e. return a new vector containing only elements for which fun returns a falsy (either false or nil ) value. Runs in linear time. Examples iex&gt; vector = A.Vector . new ( 1 .. 12 ) iex&gt; A.Vector . reject ( vector , fn i -&gt; rem ( i , 3 ) == 0 end ) vec ( [ 1 , 2 , 4 , 5 , 7 , 8 , 10 , 11 ] )","ref":"A.Vector.html#reject/2","title":"A.Vector.reject/2","type":"function"},{"doc":"Populates a vector of size n by calling generator_fun repeatedly. Examples # Although not necessary, let&#39;s seed the random algorithm iex&gt; :rand . seed ( :exrop , { 1 , 2 , 3 } ) iex&gt; A.Vector . repeat ( &amp; :rand . uniform / 0 , 3 ) vec ( [ 0.7498295129076106 , 0.06161655489244533 , 0.7924073127680873 ] )","ref":"A.Vector.html#repeat/2","title":"A.Vector.repeat/2","type":"function"},{"doc":"Returns a copy of vector with a replaced value at the specified index . Returns the vector untouched if index is out of bounds. Supports negative indexing from the end of the vector . Runs in effective constant time. Examples iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . replace_at ( 5 , :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , :foo , 7 , 8 ] ) iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . replace_at ( 8 , :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ] ) iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . replace_at ( - 2 , :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , :foo , 8 ] )","ref":"A.Vector.html#replace_at/3","title":"A.Vector.replace_at/3","type":"function"},{"doc":"Returns a copy of vector with a replaced value at the specified index . Raises an A.Vector.IndexError if index is out of bounds. Supports negative indexing from the end of the vector . Runs in effective constant time. Examples iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . replace_at! ( 5 , :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , :foo , 7 , 8 ] ) iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . replace_at! ( - 2 , :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , :foo , 8 ] ) iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . replace_at! ( 8 , :foo ) ** (A.Vector.IndexError) out of bound index: 8 not in -8..7","ref":"A.Vector.html#replace_at!/3","title":"A.Vector.replace_at!/3","type":"function"},{"doc":"Returns the vector in reverse order. Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 12 ) |&gt; A.Vector . reverse ( ) vec ( [ 12 , 11 , 10 , 9 , 8 , 7 , 6 , 5 , 4 , 3 , 2 , 1 ] )","ref":"A.Vector.html#reverse/1","title":"A.Vector.reverse/1","type":"function"},{"doc":"Returns the vector in reverse order, and concatenates the tail (enumerable). Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 5 ) |&gt; A.Vector . reverse ( 100 .. 105 ) vec ( [ 5 , 4 , 3 , 2 , 1 , 100 , 101 , 102 , 103 , 104 , 105 ] )","ref":"A.Vector.html#reverse/2","title":"A.Vector.reverse/2","type":"function"},{"doc":"Applies the given function to each element in the vector , storing the result in a vector and passing it as the accumulator for the next computation. Uses the first element in the vector as the starting value. Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . scan ( &amp; + / 2 ) vec ( [ 1 , 3 , 6 , 10 , 15 , 21 , 28 , 36 , 45 , 55 ] )","ref":"A.Vector.html#scan/2","title":"A.Vector.scan/2","type":"function"},{"doc":"Applies the given function to each element in the vector , storing the result in a vector and passing it as the accumulator for the next computation. Uses the given acc as the starting value. Runs in linear time. Examples iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . scan ( 100 , &amp; + / 2 ) vec ( [ 101 , 103 , 106 , 110 , 115 , 121 , 128 , 136 , 145 , 155 ] )","ref":"A.Vector.html#scan/3","title":"A.Vector.scan/3","type":"function"},{"doc":"Returns a new vector with the elements of vector shuffled. See Enum.shuffle/1 for notes on implementation and random seed. Examples # Although not necessary, let&#39;s seed the random algorithm iex&gt; :rand . seed ( :exrop , { 1 , 2 , 3 } ) iex&gt; A.Vector . new ( [ 1 , 2 , 3 ] ) |&gt; A.Vector . shuffle ( ) vec ( [ 3 , 1 , 2 ] ) iex&gt; A.Vector . new ( [ 1 , 2 , 3 ] ) |&gt; A.Vector . shuffle ( ) vec ( [ 1 , 3 , 2 ] )","ref":"A.Vector.html#shuffle/1","title":"A.Vector.shuffle/1","type":"function"},{"doc":"Returns the number of elements in vector . Runs in constant time. Examples iex&gt; A.Vector . new ( 10_000 .. 20_000 ) |&gt; A.Vector . size ( ) 10001 iex&gt; A.Vector . new ( ) |&gt; A.Vector . size ( ) 0","ref":"A.Vector.html#size/1","title":"A.Vector.size/1","type":"function"},{"doc":"Returns a subset of the given vector by index_range . Works the same as Enum.slice/2 , see its documentation for more details. Runs in linear time regarding the size of the returned subset. Examples iex&gt; A.Vector . new ( 0 .. 100 ) |&gt; A.Vector . slice ( 80 .. 90 ) vec ( [ 80 , 81 , 82 , 83 , 84 , 85 , 86 , 87 , 88 , 89 , 90 ] ) iex&gt; A.Vector . new ( 0 .. 100 ) |&gt; A.Vector . slice ( - 40 .. - 30 ) vec ( [ 61 , 62 , 63 , 64 , 65 , 66 , 67 , 68 , 69 , 70 , 71 ] ) iex&gt; A.Vector . new ( [ :only_one ] ) |&gt; A.Vector . slice ( 0 .. 1000 ) vec ( [ :only_one ] )","ref":"A.Vector.html#slice/2","title":"A.Vector.slice/2","type":"function"},{"doc":"Returns a subset of the given vector , from start_index (zero-based) with amount number of elements if available. Works the same as Enum.slice/3 , see its documentation for more details. Runs in linear time regarding the size of the returned subset. Examples iex&gt; A.Vector . new ( 0 .. 100 ) |&gt; A.Vector . slice ( 80 , 10 ) vec ( [ 80 , 81 , 82 , 83 , 84 , 85 , 86 , 87 , 88 , 89 ] ) iex&gt; A.Vector . new ( 0 .. 100 ) |&gt; A.Vector . slice ( - 40 , 10 ) vec ( [ 61 , 62 , 63 , 64 , 65 , 66 , 67 , 68 , 69 , 70 ] ) iex&gt; A.Vector . new ( [ :only_one ] ) |&gt; A.Vector . slice ( 0 , 1000 ) vec ( [ :only_one ] )","ref":"A.Vector.html#slice/3","title":"A.Vector.slice/3","type":"function"},{"doc":"Sorts the vector in the same way as Enum.sort/1 . Examples iex&gt; A.Vector . new ( 9 .. 1 ) |&gt; A.Vector . sort ( ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 ] )","ref":"A.Vector.html#sort/1","title":"A.Vector.sort/1","type":"function"},{"doc":"Sorts the vector in the same way as Enum.sort/2 . See Enum.sort/2 documentation for detailled usage. Examples iex&gt; A.Vector . new ( 1 .. 9 ) |&gt; A.Vector . sort ( :desc ) vec ( [ 9 , 8 , 7 , 6 , 5 , 4 , 3 , 2 , 1 ] )","ref":"A.Vector.html#sort/2","title":"A.Vector.sort/2","type":"function"},{"doc":"Sorts the vector in the same way as Enum.sort_by/3 . See Enum.sort_by/3 documentation for detailled usage. Examples iex&gt; vector = A.Vector . new ( [ &quot;some&quot; , &quot;kind&quot; , &quot;of&quot; , &quot;monster&quot; ] ) iex&gt; A.Vector . sort_by ( vector , &amp; byte_size / 1 ) vec ( [ &quot;of&quot; , &quot;some&quot; , &quot;kind&quot; , &quot;monster&quot; ] ) iex&gt; A.Vector . sort_by ( vector , &amp; { byte_size ( &amp;1 ) , String . first ( &amp;1 ) } ) vec ( [ &quot;of&quot; , &quot;kind&quot; , &quot;some&quot; , &quot;monster&quot; ] )","ref":"A.Vector.html#sort_by/3","title":"A.Vector.sort_by/3","type":"function"},{"doc":"Splits the vector into two vectors, leaving amount elements in the first one. If amount is a negative number, it starts counting from the back to the beginning of the vector . Runs in linear time. Examples iex&gt; vector = A.Vector . new ( [ 1 , 2 , 3 ] ) iex&gt; A.Vector . split ( vector , 2 ) |&gt; inspect ( ) &quot;{vec([1, 2]), vec([3])}&quot; iex&gt; A.Vector . split ( vector , 10 ) |&gt; inspect ( ) &quot;{vec([1, 2, 3]), vec([])}&quot; iex&gt; A.Vector . split ( vector , 0 ) |&gt; inspect ( ) &quot;{vec([]), vec([1, 2, 3])}&quot; iex&gt; A.Vector . split ( vector , - 1 ) |&gt; inspect ( ) &quot;{vec([1, 2]), vec([3])}&quot; iex&gt; A.Vector . split ( vector , - 5 ) |&gt; inspect ( ) &quot;{vec([]), vec([1, 2, 3])}&quot;","ref":"A.Vector.html#split/2","title":"A.Vector.split/2","type":"function"},{"doc":"Splits vector in two at the position of the element for which fun returns a falsy value ( false or nil ) for the first time. It returns a two-element tuple with two vectors of elements. The element that triggered the split is part of the second vector. Is basically performing take_while/2 and drop_while/2 at once. Runs in linear time. Examples iex&gt; { taken , dropped } = A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . split_while ( fn x -&gt; x &lt; 7 end ) iex&gt; taken vec ( [ 1 , 2 , 3 , 4 , 5 , 6 ] ) iex&gt; dropped vec ( [ 7 , 8 , 9 , 10 ] )","ref":"A.Vector.html#split_while/2","title":"A.Vector.split_while/2","type":"function"},{"doc":"Splits the vector in two vectors according to the given function fun . Returns a tuple with the first vector containing all the elements in vector for which applying fun returned a truthy value, and a second vector with all the elements for which applying fun returned a falsy value ( false or nil ). Returns the same result as filter/2 and reject/2 at once, but only walks the vector once and calls fun exactly once per element. Runs in linear time. Examples iex&gt; vector = A.Vector . new ( 1 .. 12 ) iex&gt; { filtered , rejected } = A.Vector . split_with ( vector , fn i -&gt; rem ( i , 3 ) == 0 end ) iex&gt; filtered vec ( [ 3 , 6 , 9 , 12 ] ) iex&gt; rejected vec ( [ 1 , 2 , 4 , 5 , 7 , 8 , 10 , 11 ] )","ref":"A.Vector.html#split_with/2","title":"A.Vector.split_with/2","type":"function"},{"doc":"Takes an amount of elements from the beginning or the end of the vector . If a positive amount is given, it takes the amount elements from the beginning of the vector . If a negative amount is given, the amount of elements will be taken from the end. If amount is 0, it returns the empty vector. Time complexity is: effective constant time when amount is positive, as the vector structure can be shared linear when amount is negative, as the vector needs to be reconstructed. Examples iex&gt; A.Vector . new ( 0 .. 100 ) |&gt; A.Vector . take ( 10 ) vec ( [ 0 , 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 ] ) iex&gt; A.Vector . new ( [ :only_one ] ) |&gt; A.Vector . take ( 1000 ) vec ( [ :only_one ] ) iex&gt; A.Vector . new ( 0 .. 10 ) |&gt; A.Vector . take ( - 5 ) vec ( [ 6 , 7 , 8 , 9 , 10 ] )","ref":"A.Vector.html#take/2","title":"A.Vector.take/2","type":"function"},{"doc":"Takes amount random elements from vector . Note that, unless amount is 0 or 1 , this function will traverse the whole vector to get the random sub-vector. If amount is more than the vector size, this is equivalent to shuffling the vector : the returned vector cannot be bigger than the original one. See Enum.random/1 for notes on implementation and random seed. Runs in linear time (except for amount &lt;= 1 , which is effective constant time). Examples # Although not necessary, let&#39;s seed the random algorithm iex&gt; :rand . seed ( :exrop , { 1 , 2 , 3 } ) iex&gt; A.Vector . new ( 1 .. 10 ) |&gt; A.Vector . take_random ( 2 ) vec ( [ 7 , 2 ] ) iex&gt; A.Vector . new ( [ :foo , :bar , :baz ] ) |&gt; A.Vector . take_random ( 100 ) vec ( [ :bar , :baz , :foo ] )","ref":"A.Vector.html#take_random/2","title":"A.Vector.take_random/2","type":"function"},{"doc":"Takes the elements from the beginning of the vector while fun returns a truthy value. Runs in linear time regarding the size of the returned subset. Examples iex&gt; A.Vector . new ( 1 .. 100 ) |&gt; A.Vector . take_while ( fn x -&gt; x &lt; 7 end ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 ] ) iex&gt; A.Vector . new ( [ 1 , true , %{ } , nil , &quot;abc&quot; ] ) |&gt; A.Vector . take_while ( fn x -&gt; x end ) vec ( [ 1 , true , %{ } ] )","ref":"A.Vector.html#take_while/2","title":"A.Vector.take_while/2","type":"function"},{"doc":"Converts the vector to a list. Runs in linear time. Examples iex&gt; A.Vector . new ( 10 .. 25 ) |&gt; A.Vector . to_list ( ) [ 10 , 11 , 12 , 13 , 14 , 15 , 16 , 17 , 18 , 19 , 20 , 21 , 22 , 23 , 24 , 25 ] iex&gt; A.Vector . new ( ) |&gt; A.Vector . to_list ( ) [ ]","ref":"A.Vector.html#to_list/1","title":"A.Vector.to_list/1","type":"function"},{"doc":"Returns a copy of the vector without any duplicated element. The first occurrence of each element is kept. Examples iex&gt; A.Vector . new ( [ 1 , 1 , 2 , 1 , 2 , 3 , 2 ] ) |&gt; A.Vector . uniq ( ) vec ( [ 1 , 2 , 3 ] )","ref":"A.Vector.html#uniq/1","title":"A.Vector.uniq/1","type":"function"},{"doc":"Returns a copy of the vector without elements for which the function fun returned duplicate elements. The first occurrence of each element is kept. Examples iex&gt; vector = A.Vector . new ( [ x : 1 , y : 2 , z : 1 ] ) vec ( [ x : 1 , y : 2 , z : 1 ] ) iex&gt; A.Vector . uniq_by ( vector , fn { _x , y } -&gt; y end ) vec ( [ x : 1 , y : 2 ] )","ref":"A.Vector.html#uniq_by/2","title":"A.Vector.uniq_by/2","type":"function"},{"doc":"Opposite of zip/2 . Extracts two-element tuples from the given vector and groups them together. It takes a vector with elements being two-element tuples and returns a tuple with two vectors, each of which is formed by the first and second element of each tuple, respectively. This function fails unless vector only contains tuples with exactly two elements in each tuple. Runs in linear time. iex&gt; { vector1 , vector2 } = A.Vector . new ( [ { 1 , :a } , { 2 , :b } , { 3 , :c } ] ) |&gt; A.Vector . unzip ( ) iex&gt; vector1 vec ( [ 1 , 2 , 3 ] ) iex&gt; vector2 vec ( [ :a , :b , :c ] )","ref":"A.Vector.html#unzip/1","title":"A.Vector.unzip/1","type":"function"},{"doc":"Returns a copy of vector with an updated value at the specified index . Returns the vector untouched if index is out of bounds. Supports negative indexing from the end of the vector . Runs in effective constant time. Examples iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . update_at ( 2 , &amp; ( &amp;1 * 1000 ) ) vec ( [ 1 , 2 , 3000 , 4 , 5 , 6 , 7 , 8 ] ) iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . update_at ( 8 , &amp; ( &amp;1 * 1000 ) ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 ] ) iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . update_at ( - 1 , &amp; ( &amp;1 * 1000 ) ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8000 ] )","ref":"A.Vector.html#update_at/3","title":"A.Vector.update_at/3","type":"function"},{"doc":"Returns a copy of vector with an updated value at the specified index . Raises an A.Vector.IndexError if index is out of bounds. Supports negative indexing from the end of the vector . Runs in effective constant time. Examples iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . update_at! ( 2 , &amp; ( &amp;1 * 1000 ) ) vec ( [ 1 , 2 , 3000 , 4 , 5 , 6 , 7 , 8 ] ) iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . update_at! ( - 1 , &amp; ( &amp;1 * 1000 ) ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8000 ] ) iex&gt; A.Vector . new ( 1 .. 8 ) |&gt; A.Vector . update_at! ( - 9 , &amp; ( &amp;1 * 1000 ) ) ** (A.Vector.IndexError) out of bound index: -9 not in -8..7","ref":"A.Vector.html#update_at!/3","title":"A.Vector.update_at!/3","type":"function"},{"doc":"Returns the vector with each element wrapped in a tuple alongside its index. May receive a function or an integer offset. If an integer offset is given, it will index from the given offset instead of from zero. If a function is given, it will index by invoking the function for each element and index (zero-based) of the vector . Runs in linear time. Examples iex&gt; vector = A.Vector . new ( [ &quot;foo&quot; , &quot;bar&quot; , &quot;baz&quot; ] ) iex&gt; A.Vector . with_index ( vector ) vec ( [ { &quot;foo&quot; , 0 } , { &quot;bar&quot; , 1 } , { &quot;baz&quot; , 2 } ] ) iex&gt; A.Vector . with_index ( vector , 100 ) vec ( [ { &quot;foo&quot; , 100 } , { &quot;bar&quot; , 101 } , { &quot;baz&quot; , 102 } ] ) iex&gt; A.Vector . with_index ( vector , fn element , index -&gt; { index , element } end ) vec ( [ { 0 , &quot;foo&quot; } , { 1 , &quot;bar&quot; } , { 2 , &quot;baz&quot; } ] )","ref":"A.Vector.html#with_index/2","title":"A.Vector.with_index/2","type":"function"},{"doc":"Zips corresponding elements from two vectors into one vector of tuples. The size of the returned vector is the one of the smallest of the input vectors. Runs in linear time. iex&gt; A.Vector . zip ( A.Vector . new ( [ 1 , 2 , 3 ] ) , A.Vector . new ( [ :a , :b , :c ] ) ) vec ( [ { 1 , :a } , { 2 , :b } , { 3 , :c } ] ) iex&gt; A.Vector . zip ( A.Vector . new ( 0 .. 100 ) , A.Vector . new ( [ :a , :b , :c ] ) ) vec ( [ { 0 , :a } , { 1 , :b } , { 2 , :c } ] )","ref":"A.Vector.html#zip/2","title":"A.Vector.zip/2","type":"function"},{"doc":"Zips corresponding elements from two vectors into a new vector, transforming them with the zip_fun function as it goes. The corresponding elements from each vector are passed to the provided 2-arity zip_fun function in turn. Runs in linear time. iex&gt; A.Vector . zip_with ( A.Vector . new ( [ 1 , 2 , 3 ] ) , A.Vector . new ( [ :a , :b , :c ] ) , &amp; { &amp;2 , &amp;1 } ) vec ( [ a : 1 , b : 2 , c : 3 ] ) iex&gt; A.Vector . zip_with ( A.Vector . new ( 0 .. 100 ) , A.Vector . new ( [ :a , :b , :c ] ) , &amp; { &amp;2 , &amp;1 } ) vec ( [ a : 0 , b : 1 , c : 2 ] )","ref":"A.Vector.html#zip_with/3","title":"A.Vector.zip_with/3","type":"function"},{"doc":"","ref":"A.Vector.html#t:index/0","title":"A.Vector.index/0","type":"type"},{"doc":"","ref":"A.Vector.html#t:t/0","title":"A.Vector.t/0","type":"type"},{"doc":"","ref":"A.Vector.html#t:t/1","title":"A.Vector.t/1","type":"opaque"},{"doc":"","ref":"A.Vector.html#t:value/0","title":"A.Vector.value/0","type":"type"},{"doc":"","ref":"A.Vector.EmptyError.html","title":"A.Vector.EmptyError","type":"exception"},{"doc":"","ref":"A.Vector.IndexError.html","title":"A.Vector.IndexError","type":"exception"},{"doc":"Extension of the Elixir standard library focused on data stuctures, data manipulation and performance. Data structures Utility functions Installation About Aja FAQ","ref":"readme.html","title":"README","type":"extras"},{"doc":"&quot;there is one aspect of functional programming that no amount of cleverness on the part of the compiler writer is likely to mitigate — the use of inferior or inappropriate data structures.&quot; -- Chris Okasaki Persistent vectors: A.Vector A blazing fast, pure Elixir implementation of a persistent vector, meant to offer an efficient alternative to lists. Supports many operations like appends and random access in effective constant time. iex&gt; vector = A.Vector . new ( 1 .. 10 ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 ] ) iex&gt; A.Vector . append ( vector , :foo ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , 10 , :foo ] ) iex&gt; vector [ 3 ] 4 iex&gt; A.Vector . replace_at ( vector , - 1 , :bar ) vec ( [ 1 , 2 , 3 , 4 , 5 , 6 , 7 , 8 , 9 , :bar ] ) iex&gt; 3 in vector true A.Vector reimplements many of the functions from the Enum module specifically for vectors, with efficiency in mind. It should be easier to use from Elixir than Erlang's :array module and faster in most cases. The A.vec/1 and A.vec_size/1 macros, while being totally optional, can make it easier to work with vectors and make pattern-matching possible: iex&gt; import A iex&gt; vec ( [ a , 2 , c , _d , e ] ) = A.Vector . new ( 1 .. 5 ) ; { a , c , e } { 1 , 3 , 5 } iex&gt; vec ( first ||| last ) = A.Vector . new ( 1 .. 1_000_000 ) ; { first , last } { 1 , 1000000 } iex&gt; match? ( v when vec_size ( v ) &gt; 9 , vec ( 1 .. 10 ) ) true The A.+++/2 operator provides synctactic sugar for vector concatenation: iex&gt; vec ( [ 1 , 2 , 3 ] ) ++ + vec ( [ 4 , 5 ] ) vec ( [ 1 , 2 , 3 , 4 , 5 ] ) Ordered maps: A.OrdMap The standard library does not offer any similar functionality: regular maps do not keep track of the insertion order keywords do but they only support atoms and do not have the right performance characteristics (plain lists) iex&gt; %{ &quot;one&quot; =&gt; 1 , &quot;two&quot; =&gt; 2 , &quot;three&quot; =&gt; 3 } %{ &quot;one&quot; =&gt; 1 , &quot;three&quot; =&gt; 3 , &quot;two&quot; =&gt; 2 } iex&gt; ord_map = A.OrdMap . new ( [ { &quot;one&quot; , 1 } , { &quot;two&quot; , 2 } , { &quot;three&quot; , 3 } ] ) ord ( %{ &quot;one&quot; =&gt; 1 , &quot;two&quot; =&gt; 2 , &quot;three&quot; =&gt; 3 } ) iex&gt; ord_map [ &quot;two&quot; ] 2 iex&gt; Enum . to_list ( ord_map ) [ { &quot;one&quot; , 1 } , { &quot;two&quot; , 2 } , { &quot;three&quot; , 3 } ] Ordered maps behave pretty much like regular maps, and the A.OrdMap module offers the same API as Map . The convenience macro A.ord/1 make them a breeze to instantiate or pattern-match upon: iex&gt; import A iex&gt; ord_map = ord ( %{ &quot;一&quot; =&gt; 1 , &quot;二&quot; =&gt; 2 , &quot;三&quot; =&gt; 3 } ) ord ( %{ &quot;一&quot; =&gt; 1 , &quot;二&quot; =&gt; 2 , &quot;三&quot; =&gt; 3 } ) iex&gt; ord ( %{ &quot;三&quot; =&gt; three , &quot;一&quot; =&gt; one } ) = ord_map iex&gt; { one , three } { 1 , 3 } All data structures offer: great performance characteristics at any size (see FAQ ) well-documented APIs that are consistent with the standard library implementation of Inspect , Enumerable and Collectable protocols implementation of the Access behaviour (optional if Jason is installed) implemention of the Jason.Encoder protocol Optimized Enum : A.Enum A.Enum mirrors the Enum module, but its implementation is highly optimized for Aja structures such as A.Vector or A.OrdMap . A.Enum on vectors/ord maps can often be faster than Enum on lists/maps, depending on the function and size of the sequence.","ref":"readme.html#data-structures","title":"README - Data structures","type":"extras"},{"doc":"IO data IO data are nested structures based on lists to work more efficiently with binary/text data without the need for any expensive concatenation. The ~i sigil provides a way to build IO data using string interpolation: iex&gt; import A iex&gt; ~i&quot;atom: \#{ :foo } , charlist: \#{ &#39;abc&#39; } , number: \#{ 12 + 2.35 } \\n &quot; [ &quot;atom: &quot; , &quot;foo&quot; , &quot;, charlist: &quot; , &#39;abc&#39; , &quot;, number: &quot; , &quot;14.35&quot; , 10 ] The A.IO module provides functions to work with IO data: iex&gt; A.IO . to_iodata ( :foo ) &quot;foo&quot; iex&gt; A.IO . to_iodata ( [ &quot;abc&quot; , &#39;def&#39; | &quot;ghi&quot; ] ) [ &quot;abc&quot; , &#39;def&#39; | &quot;ghi&quot; ] iex&gt; A.IO . iodata_empty? ( [ &quot;&quot; , [ ] ] ) true","ref":"readme.html#utility-functions","title":"README - Utility functions","type":"extras"},{"doc":"Aja can be installed by adding aja to your list of dependencies in mix.exs : def deps do [ { :aja , &quot;~&gt; 0.5.3&quot; } ] end Or, if you are using Elixir 1.12, you can just try it out from iex or an .exs script: iex&gt; Mix . install ( [ :aja ] ) :ok iex&gt; A.Vector . new ( [ &quot;Hello&quot; , &quot;world!&quot; ] ) vec ( [ &quot;Hello&quot; , &quot;world!&quot; ] ) Documentation can be found at https://hexdocs.pm/aja .","ref":"readme.html#installation","title":"README - Installation","type":"extras"},{"doc":"Inspirations the amazingly polished Elixir standard library : self-consistent, well-documented and just delightful ✨️ the also amazing Python standard library , notably its collections module various work on efficient persistent data structures spearheaded by Okasaki Clojure's persistent vectors, by Rich Hickey and influenced by Phil Bagwell Goals being consistent with Elixir and with itself (API, quality, documentation) no external dependency to help you preserve a decent dependency tree performance-conscious (right algorithm, proper benchmarking, fast compile times*) mostly dead-simple pure functions: no configuration, no mandatory macro, no statefulness / OTP (* while fast compile time is a target, vectors are optimized for fast runtime at the expense of compile time) Resources Chris Okasaki's Purely Functional Data Structures Jean Niklas L'orange's articles and thesis about persistent vectors and RRB trees","ref":"readme.html#about-aja","title":"README - About Aja","type":"extras"},{"doc":"How stable is it? Aja is still pretty early stage and the high-level organisation is still in flux. Expect some breaking changes until it reaches maturity. However, most of its APIs are based on the standard library and should therefore remain fairly stable. Besides, Aja is tested quite thoroughly both with unit tests and property-based testing (especially for data structures). This effort is far from perfect, but increases our confidence in the overall reliability. How is the performance? Vectors Most operations from A.Vector are much faster than Erlang's :array equivalents, and in some cases are even noticeably faster than equivalent list operations (map, folds, join, sum...). Make sure to read the efficiency guide from A.Vector doc. Ordered maps Performance for ordered maps has an inevitable though decent overhead over plain maps in terms of creation and update time (write operations), as well as memory usage, since some extra work is needed to keep track of the order. It has however very good read performance, with a very minimal overhead in terms of key access, and can be enumerated much faster than maps using A.Enum . Aja 💖️ JIT Aja's data structures (vectors and ordered maps) are already pretty fast on pre-JIT versions of OTP ( &lt;= 23 ). Benchmarks on OTP 24 suggest however that they are taking great advantage of the JIT , relative to lists/maps, making them even more interesting performance-wise. Benchmarks Aja data structures should work fine in most cases, but if you're considering them for performance-critical sections of your code, make sure to benchmark them. Benchmarking is still a work in progress, but you can check the bench folder for more detailed figures.","ref":"readme.html#faq","title":"README - FAQ","type":"extras"},{"doc":"Aja is licensed under the MIT License .","ref":"readme.html#copyright-and-license","title":"README - Copyright and License","type":"extras"},{"doc":"Changelog","ref":"changelog.html","title":"Changelog","type":"extras"},{"doc":"","ref":"changelog.html#dev","title":"Changelog - Dev","type":"extras"},{"doc":"Enhancement Optimize A.OrdMap internals to reduce memory usage Bug fixes Fix compilation warning in deprecated module","ref":"changelog.html#v0-5-3-2021-08-20","title":"Changelog - v0.5.3 (2021-08-20)","type":"extras"},{"doc":"Enhancement Add support for stepped ranges in Elixir 1.12 More efficient implementation of vector concatenation Add A.Enum.concat/1 and A.Enum.into/3 Add A.Vector.zip_with/3 Bug fixes Fix bug in A.Enum.into/2 when merging into an A.OrdMap Fix A.vec/1 bug when ranges use negative number Breaking changes Deprecate A.ExRange","ref":"changelog.html#v0-5-2-2021-07-01","title":"Changelog - v0.5.2 (2021-07-01)","type":"extras"},{"doc":"Enhancements Add A.Vector.flat_map/2 Add A.Vector.reverse/2 and A.Enum.reverse/2 Add A.Enum.concat/2 Add A.Enum.zip/2 and A.Enum.unzip/1 A.OrdMap.drop/2 will only try to period rebuild once after dropping all keys Bug fixes Fix the minimal version needed for Jason (1.2) A.OrdMap.take/2 behaves as expected when keys are duplicated A.ord/1 warning on duplicate keys uses the proper stacktrace Breaking changes Change Inspect protocol for A.Vector and A.OrdMap Deprecate A.Enum.sort_uniq/1, A.Enum.sort_uniq/2","ref":"changelog.html#v0-5-1-2021-04-09","title":"Changelog - v0.5.1 (2021-04-09)","type":"extras"},{"doc":"Enhancements A.OrdMap new implemention, with highly improved performance ⚡️⚡️ A.Enum as a faster Enum module optimized for Aja structures (vectors, ord maps) Added A.ord_size/1 macro A.vec/1 can pattern-match on first and last elements A.ord/1 warns on duplicate errors and can generate the AST on compile time for constant keys Breaking changes Reimplement A.OrdMap , changing its internals completely Change signature of A.OrdMap.foldl/3 and A.OrdMap.foldr/3 for consistency Remove A.OrdMap.pop_first/1 and A.OrdMap.pop_last/1 Modify A.Vector internals (only breaking if persisted) Deprecate methods from A.Vector that have been moved to A.Enum Remove A.RBMap and A.RBSet Rename A.List.repeat/2 and A.Vector.repeat/2 (previously &quot;repeatedly&quot;)","ref":"changelog.html#v0-5-0-2021-03-25","title":"Changelog - v0.5.0 (2021-03-25)","type":"extras"},{"doc":"Enhancements Add A.Vector.map_reduce/3 Add A.Vector.scan/2 and A.Vector.scan/3 A.Vector.with_index/2 accepts a function parameter Breaking changes Deprecate A.RBMap and A.RBSet Bug fixes Fix A.Vector.join/2 bug when working with chardata","ref":"changelog.html#v0-4-8-2021-02-23","title":"Changelog - v0.4.8 (2021-02-23)","type":"extras"},{"doc":"Enhancements Add A.Vector.reduce/2 and A.Vector.reduce/3 Add A.Vector.split/2 and A.Vector.split_with/2 Add A.Vector.frequencies/1 and A.Vector.frequencies_by/2 Add A.Vector.group_by/3 Add A.Vector.dedup/1 and A.Vector.dedup_by/2 Add A.Vector.min/2, A.Vector.max/2, A.Vector.min_by/3, A.Vector.max_by/3 Improve performance for A.Vector.min/1, A.Vector.max/1 and A.Vector.uniq/1","ref":"changelog.html#v0-4-7-2021-02-19","title":"Changelog - v0.4.7 (2021-02-19)","type":"extras"},{"doc":"Enhancements Add A.Vector.find_index/2 Add A.Vector.take_while/2 , A.Vector.drop_while/2 and A.Vector.split_while/2 Add A.Vector.zip/2 and A.Vector.unzip/1 Add A.Vector.fetch!/2 alias","ref":"changelog.html#v0-4-6-2021-02-10","title":"Changelog - v0.4.6 (2021-02-10)","type":"extras"},{"doc":"Enhancements Add A.Vector.find/3 and A.Vector.find_value/3 Add A.+++/2 convenience operator Improve efficiency of vector access functions: A.Vector.at/2 , A.Vector.fetch/2 , A.Vector.replace_at/3 , A.Vector.delete_at/2 ... ⚡️ Improve compile times Bug fixes Invoke callbacks in the right order for: A.Vector.filter/2 , A.Vector.reject/2 and improve performance","ref":"changelog.html#v0-4-5-2021-01-31","title":"Changelog - v0.4.5 (2021-01-31)","type":"extras"},{"doc":"Enhancements Add A.vec_size/1 macro Add A.Vector.with_index/2 Add A.Vector.random/1, A.Vector.take_random/2 and A.Vector.shuffle/1 Drastically improve efficiency of A.Vector.duplicate/2 ⚡️⚡️⚡️ Breaking changes Changed internal representation of A.Vector (only breaking if persisted) Stop documenting and exposing internal trees (A.RBTree) Rename and deprecate A.Vector.append_many/2 to A.Vector.concat/2","ref":"changelog.html#v0-4-4-2021-01-23","title":"Changelog - v0.4.4 (2021-01-23)","type":"extras"},{"doc":"This release is mostly focused on vector slicing and performance ⚡️ Enhancements Add A.Vector.each/2 Add A.Vector.slice/2 , A.Vector.slice/3 , A.Vector.take/2 and A.Vector.drop/2 A.Vector efficiently implements Enumerable.slice/1 Reimplement A.Vector.delete_at/2 , A.Vector.pop_at/2 efficiently","ref":"changelog.html#v0-4-3-2021-01-12","title":"Changelog - v0.4.3 (2021-01-12)","type":"extras"},{"doc":"Enhancements Implement A.Vector.product/1 Bug fixes Invoke callbacks in the right order for: A.Vector.map/2 , A.Vector.map_intersperse/3 , A.Vector.map_join/3, A.Vector.any?/2, A.Vector.all?/2","ref":"changelog.html#v0-4-2-2021-01-10","title":"Changelog - v0.4.2 (2021-01-10)","type":"extras"},{"doc":"Enhancements Add A.Vector.map_join/3 and A.Vector.map_intersperse/3 Implement A.Vector.intersperse/2 more efficiently Bug fixes A.Vector.sum/1 adds using the same order as Enum.sum/1 , avoiding slight inconsistencies for floats","ref":"changelog.html#v0-4-1-2020-12-05","title":"Changelog - v0.4.1 (2020-12-05)","type":"extras"},{"doc":"Enhancements Introduce persistent vectors: A.Vector 🚀️ Add A.vec/1","ref":"changelog.html#v0-4-0-2020-12-02","title":"Changelog - v0.4.0 (2020-12-02)","type":"extras"},{"doc":"Enhancements Add A.sigil_i/2 Add A.IO.to_iodata/1","ref":"changelog.html#v0-3-3-2020-11-14","title":"Changelog - v0.3.3 (2020-11-14)","type":"extras"},{"doc":"Enhancements Add A.Enum.sort_uniq/1, A.Enum.sort_uniq/2 Add A.List.prepend/2","ref":"changelog.html#v0-3-2-2020-11-14","title":"Changelog - v0.3.2 (2020-11-14)","type":"extras"},{"doc":"Bug fixes A.RBMap.Enumerable.member?/2 returns false instead of crashing for values other than size-2 tuples","ref":"changelog.html#v0-3-1-2020-11-05","title":"Changelog - v0.3.1 (2020-11-05)","type":"extras"},{"doc":"Enhancements Rework all internals, improved peformance for A.OrdMap , A.RBMap, A.RBSet Add default parameter to A.OrdMap.first/1 , A.OrdMap.last/1 , A.RBMap.first/1, A.RBMap.last/1, A.RBSet.first/1, A.RBSet.last/1 Breaking changes Changed signature of A.OrdMap.foldl/3 , A.OrdMap.foldr/3 , A.RBMap.foldl/3, A.RBMap.foldr/3 Internals of all data structures have been changed Split A.RBTree as A.RBTree.Map and A.RBTree.Set","ref":"changelog.html#v0-3-0-2020-10-31","title":"Changelog - v0.3.0 (2020-10-31)","type":"extras"},{"doc":"Enhancements Add A.String.slugify/2 Breaking changes Remove A.Array module","ref":"changelog.html#v0-2-0-2020-10-25","title":"Changelog - v0.2.0 (2020-10-25)","type":"extras"},{"doc":"Enhancements Add pop_first/1 and pop_last/1 to A.RBMap, A.RBSet and A.OrdMap Add some guards to functions Bug fixes A.RBSet.disjoint?/2 was not returning the expected value","ref":"changelog.html#v0-1-2-2020-10-22","title":"Changelog - v0.1.2 (2020-10-22)","type":"extras"},{"doc":"Bug fixes Fix incompatibility with Elixir 1.10","ref":"changelog.html#v0-1-1-2020-10-21","title":"Changelog - v0.1.1 (2020-10-21)","type":"extras"},{"doc":"Initial release","ref":"changelog.html#v0-1-0-2020-10-18","title":"Changelog - v0.1.0 (2020-10-18)","type":"extras"},{"doc":"MIT License Copyright (c) 2020 Sabiwara Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the &quot;Software&quot;), to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions: The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software. THE SOFTWARE IS PROVIDED &quot;AS IS&quot;, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.","ref":"license.html","title":"LICENSE","type":"extras"}]